package com.sogou.shortvideo

import android.app.Activity
import android.content.Intent
import android.os.Handler
import android.os.Looper
import android.support.v4.app.FragmentActivity
import android.util.Log
import android.view.WindowManager
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.PluginRegistry

class OperationPluginImpl : IOperationPlugin{

    private val ACTION_NAVIGATOR = "action_navigator"

    private val ACTION_STATISTICS = "action_statistics"

    private val ACTION_WHEN_CRASH = "action_when_crash";

    private val ACTION_GETSYSINFO = "action_getsysinfo"

    private val  ACTION_PAGECLOSE = "action_pageClose" //页面关闭

    private val ACTION_FEEDBACK = "action_feedBack" //反馈

    private val ACTION_NOTLIKE = "action_notLike" //不感兴趣

    private val ACTION_GIVELIKE = "action_giveLike" //点赞

    private val ACTION_SHOWCOMMENTS = "action_showComments" //展示评论

    private val ACTION_WRITECOMMENTS = "action_writeComments" //写评论

    private val ACTION_OPENADDETAIL = "action_openAdDetail" //打开广告详情

    private val ACTION_FAVORITE = "action_favorite" //收藏

    private val ACTION_DOWNLOADADAPP = "action_downloadAdApp" //下载广告APP

    private val ACTION_SHARE = "action_share" //分享

    private val ACTION_SWITCH_VIDEO = "action_switch_video" //视频切换

    private val ACTION_EDUCATION_SHOW = "action_education_show" //视频教育页在展示

    private val ACTION_CACHE_VIDEO = "action_cache_video" //缓存videoplayer

    private val ACTION_STAYSCREEN_LIGHT = "action_stayscreen_light"  //保持屏幕长亮或者恢复正常

    private var activity: Activity

    constructor(activity: Activity){
        this.activity = activity
    }

    override fun dispatchCall(registrar: PluginRegistry.Registrar, call: MethodCall, result: MethodChannel.Result) {

        when(call.method){

            ACTION_NAVIGATOR -> {

                var map : HashMap<String, String> = call.arguments as HashMap<String, String>

                var it = Intent(activity, MainActivity::class.java)
                it.putExtra("args", map["scheme"])
                activity.startActivity(it)
                result.success(true)
            }

            ACTION_STATISTICS ->{

                var map : HashMap<String, Any> = call.arguments as HashMap<String, Any>
                var type : Int = map["type"] as Int
                Log.e("ACTION_STATISTICS", ""+type)
                result.success(true)
            }

            ACTION_WHEN_CRASH ->{

                var map : HashMap<String, String> = call.arguments as HashMap<String, String>
                var extra : String? = map["extra"]
                var trace : String? = map["trace"]
                Log.e("crash", extra)
                Log.e("crash", trace)
                result.success(true)
            }

            ACTION_GETSYSINFO ->{

                var map : HashMap<String, Any> = HashMap()
                map["nt"] = Utils.checkNetwork()
                map["imei"] = Utils.getDeviceIMEI()
                map.putAll(Utils.getHRV())

                result.success(map)
            }

            ACTION_PAGECLOSE ->{
                if (activity != null && !activity.isFinishing){
                    activity.finish()
                }
                Handler(Looper.getMainLooper()).postDelayed({
                    result.success(true)
                }, 150)
            }

            ACTION_GIVELIKE ->{
                result.success(1)
            }
            
            ACTION_SHARE ->{
                result.success(true)
            }
            
            ACTION_SHOWCOMMENTS ->{
                result.success(HashMap<String, Any>())
            }
            
            ACTION_WRITECOMMENTS ->{
                result.success(HashMap<String, Any>())
            }
            
            ACTION_NOTLIKE ->{
                result.success("")
            }
            
            ACTION_SWITCH_VIDEO ->{
                result.success(true)
            }

            ACTION_FEEDBACK ->{
                result.success(1)
            }

            ACTION_OPENADDETAIL ->{
                result.success(true)
            }

            ACTION_FAVORITE ->{
                result.success(true)
            }

            ACTION_DOWNLOADADAPP ->{
                result.success(true)
            }

            ACTION_EDUCATION_SHOW ->{
                result.success(true)
            }

            ACTION_CACHE_VIDEO ->{

                var map : HashMap<String, Any> = call.arguments as HashMap<String, Any>

                var append: Boolean = map["append"] as Boolean
                var urls: ArrayList<String> = map["urls"] as ArrayList<String>
//                SgVideoPlayerPlugin2.cacheVideo(append, urls)

                result.success(0)
            }

            /**
             * 保持屏幕长亮或者恢复正常
             */
            ACTION_STAYSCREEN_LIGHT -> {
                val isKeepScreenOn = call.arguments as Boolean
                val act = activity as FragmentActivity
                keepScreenOnOrNormal(isKeepScreenOn,act)
                result.success(0)
            }
        }

    }

    /**
     * 保持屏幕常亮或回复正常
     */
    private fun keepScreenOnOrNormal(isKeepScreenOn: Boolean, act: FragmentActivity) {
        if (act == null ) return
        if (isKeepScreenOn) {
            act.window.addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON)
        } else {
            act.window.clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON)
        }
    }
}