// Copyright 2017 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

package com.sogou.shortvideo

import android.net.Uri
import android.util.Log
import android.view.Surface
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import io.flutter.view.TextureRegistry
import tv.danmaku.ijk.media.player.IjkMediaPlayer
import java.util.*
import kotlin.collections.HashMap
import kotlin.collections.LinkedHashMap

class SgVideoPlayerPlugin2 : IVideoPlugin {

    companion object{
        private val cachePlayers: HashMap<String, VideoPlayer> = LinkedHashMap()

        private val videoUrls: LinkedList<String> = LinkedList()

        fun cacheVideo(append: Boolean, urls: ArrayList<String>){
            if (!append){
                videoUrls.clear()
                cachePlayers.clear()
            }

            videoUrls.addAll(urls)

            urls.forEach {
                cachePlayers[it] = VideoPlayer(it)
            }
        }
    }

    private class VideoPlayer internal constructor(
            dataSource: String?) {

        private var ijkMediaPlayer: IjkMediaPlayer? = null

        private var eventChannel: EventChannel? = null

        private var textureEntry: TextureRegistry.SurfaceTextureEntry? = null

        private var surface: Surface? = null

        private var eventSink: QueuingEventSink? = null

        private var isInitialized = false

        private var firstRenderPause = true

        private var manualSeek = false

        internal val position: Long
            get() {
                return ijkMediaPlayer!!.currentPosition
            }

        init {
            ijkMediaPlayer = IjkMediaPlayer()

            ijkMediaPlayer?.also { player->

                //视频帧处理不过来的时候丢弃一些帧达到同步的效果1-同步 0-不同步
                player.setOption(IjkMediaPlayer.OPT_CATEGORY_PLAYER, "framedrop", 1)
                //需要准备好后自动播放0-不自动 1-自动
                player.setOption(IjkMediaPlayer.OPT_CATEGORY_PLAYER, "start-on-prepared", 1)
                //最大缓存空间1M
                player.setOption(IjkMediaPlayer.OPT_CATEGORY_PLAYER, "max-buffer-size", 1*1024)
                //默认最小帧数3
                player.setOption(IjkMediaPlayer.OPT_CATEGORY_PLAYER, "min-frames", 3)
                // 播放前的探测Size
                player.setOption(IjkMediaPlayer.OPT_CATEGORY_FORMAT,"probesize",32)
                // 设置播放前的探测时间
                player.setOption(IjkMediaPlayer.OPT_CATEGORY_FORMAT,"analyzeduration",1)
                // 不查询stream_info，直接使用
                player.setOption(IjkMediaPlayer.OPT_CATEGORY_PLAYER,"find_stream_info", 0)
                // 等待开始之后才绘制
                player.setOption(IjkMediaPlayer.OPT_CATEGORY_PLAYER, "render-wait-start", 0)
                // 跳过循环滤波
                player.setOption(IjkMediaPlayer.OPT_CATEGORY_CODEC, "skip_loop_filter", 48)
                // 通过立即清理数据包来减少等待时长
                player.setOption(IjkMediaPlayer.OPT_CATEGORY_FORMAT, "flush_packets", 1)
                // 暂停输出直到停止后读取足够的数据包
                player.setOption(IjkMediaPlayer.OPT_CATEGORY_PLAYER, "packet-buffering", 0)

                Log.e("SgVideoPlayerPlugin", "dataSource=$dataSource")

                try {
                    val uri = Uri.parse(dataSource)
                    player.setDataSource(MyApp.instance, uri)
                    setupVideoPlayer()
                } catch (e: Exception) {
                    Log.i("flutter", "Exception: ${e.stackTrace}")
                    /*val event = HashMap<String, Any>()
                    event["event"] = "occurError"
                    event["error"] = "Video player init failed"
                    eventSink.success(event)

                    val reply = HashMap<String, Any>()
                    reply["textureId"] = textureEntry.id()
                    result.success(reply)*/
                }
            }
        }


        internal fun setEventChannel(eventChannel: EventChannel) {
            this.eventChannel = eventChannel
            this.eventSink = QueuingEventSink()
            eventChannel.setStreamHandler(
                    object : EventChannel.StreamHandler {
                        override fun onListen(o: Any?, sink: EventChannel.EventSink) {
                            eventSink?.setDelegate(sink)
                        }

                        override fun onCancel(o: Any?) {
                            eventSink?.setDelegate(null)
                        }
                    })
        }

        internal fun setSurface(textureEntry: TextureRegistry.SurfaceTextureEntry){
            this.textureEntry = textureEntry
            this.surface = Surface(textureEntry.surfaceTexture())
            ijkMediaPlayer?.apply {
                this.setSurface(surface)
            }
        }

        private fun setupVideoPlayer() {
            ijkMediaPlayer?.also { player->

                player.setOnErrorListener { iMediaPlayer, i, i1 ->
                    Log.i("flutter", "onError ...")
                    val event = HashMap<String, Any>()
                    event["event"] = "occurError"
                    event["error"] = "Video player had error $i, $i1"
                    eventSink?.success(event)
                    true
                }

                //非循环播放回调Completion完成事件
                player.setOnCompletionListener {
                    Log.i("flutter", "onCompletion ...")
                    val event = HashMap<String, Any>()
                    event["event"] = "completed"
                    eventSink?.success(event)
                }

                //循环播放回调SeekComplete完成事件
                player.setOnSeekCompleteListener {
                    if (!manualSeek){
                        Log.i("flutter", "onSeekComplete ...")
                        val event = HashMap<String, Any>()
                        event["event"] = "completed"
                        eventSink?.success(event)
                    }
                    manualSeek = false
                }

                player.setOnBufferingUpdateListener { iMediaPlayer, i ->
                    Log.i("flutter", "onBufferingUpdate ...")
                    sendBufferingUpdate()
                }
                player.setOnPreparedListener {
                    Log.i("flutter", "onPrepared ...")
                    if (!isInitialized) {
                        ijkMediaPlayer!!.start()
                        isInitialized = true
                        sendInitialized()
                    }
                }

                player.setOnFirstFrameRenderListener { mp ->
                    if (firstRenderPause) {
                        mp.pause()
                    }
                }

                player.prepareAsync()
            }
        }

        fun sendBufferingUpdate() {
            val event = HashMap<String, Any>()
            event["event"] = "bufferingUpdate"
            val range = Arrays.asList<Number>(0, ijkMediaPlayer!!.currentPosition)
            // iOS supports a list of buffered ranges, so here is a list with a single range.
            event["values"] = listOf(range)
            eventSink?.success(event)
        }

        internal fun play() {
            firstRenderPause = false
            ijkMediaPlayer!!.start()
        }

        internal fun pause() {
            ijkMediaPlayer!!.pause()
        }

        internal fun setLooping(value: Boolean) {
            ijkMediaPlayer!!.isLooping = value
        }

        internal fun setVolume(value: Double) {
            val bracketedValue = Math.max(0.0, Math.min(1.0, value)).toFloat()
            ijkMediaPlayer!!.setVolume(bracketedValue, bracketedValue)
        }

        internal fun seekTo(location: Int) {
            manualSeek = true
            ijkMediaPlayer!!.seekTo(location.toLong())
        }

        internal fun debug(debugCode: Int) {
            if(debugCode == 1){
                val event = HashMap<String, Any>()
                event["event"] = "occurError"
                event["error"] = "Video player had error manullay"
                eventSink?.success(event)
            }
        }

        private fun sendInitialized() {
            if (isInitialized) {
                val event = HashMap<String, Any>()
                event["event"] = "initialized"
                event["duration"] = ijkMediaPlayer!!.duration

                event["width"] = ijkMediaPlayer!!.videoWidth
                event["height"] = ijkMediaPlayer!!.videoHeight

                eventSink?.success(event)
            }
        }

        internal fun getInitializedInfo(): HashMap<String, Any>?{
            var event: HashMap<String, Any>? = null
            if (isInitialized) {
                event = HashMap()
                event["event"] = "initialized"
                event["duration"] = ijkMediaPlayer!!.duration

                event["width"] = ijkMediaPlayer!!.videoWidth
                event["height"] = ijkMediaPlayer!!.videoHeight
            }
            return event
        }

        internal fun dispose() {
            if (isInitialized) {
                ijkMediaPlayer!!.pause()
            }
            textureEntry?.release()
            eventChannel?.setStreamHandler(null)
            if (surface != null) {
                surface!!.release()
            }
//            if (ijkMediaPlayer != null) {
//                ijkMediaPlayer!!.release()
//            }
        }

        internal fun recycle() {
            if (isInitialized) {
                ijkMediaPlayer!!.stop()
            }
            textureEntry?.release()
            eventChannel?.setStreamHandler(null)
            if (surface != null) {
                surface!!.release()
            }
            if (ijkMediaPlayer != null) {
                ijkMediaPlayer!!.release()
            }
        }
    }

    private fun disposeAllPlayers() {
        cachePlayers.forEach {
            var player = it.value
            player.recycle()
        }
    }

    override fun onCreate(registrar: Registrar) {
    }
    
    override fun onDestroy() {
        // The whole FlutterView is being destroyed. Here we release resources acquired for all instances
        // of VideoPlayer. Once https://github.com/flutter/flutter/issues/19358 is resolved this may
        // be replaced with just asserting that videoPlayers.isEmpty().
        // https://github.com/flutter/flutter/issues/20989 tracks this.
        disposeAllPlayers()
    }

    override fun dispatchCall(registrar: Registrar, call: MethodCall, result: Result) {
        val textures = registrar.textures()
        if (textures == null) {
            result.error("no_activity", "video_player plugin requires a foreground activity", null)
            return
        }
        when (call.method) {
//            "init" -> disposeAllPlayers()
            "create" -> {
                val handle = textures.createSurfaceTexture()


                val eventChannel = EventChannel(
                        registrar.messenger(), "sogou.flutter/videoPlayer/videoEvents" + handle.id())

                val dataSource :String = call.argument<String>("uri")?:""

                val player: VideoPlayer?
                if (cachePlayers.containsKey(dataSource)){
                    player = cachePlayers[dataSource]
                    Log.e("SgVideoPlayerPlugin", "cache " + handle.id() + "-->" + call.method)
                }else{
                    player = VideoPlayer(dataSource)
                    cachePlayers[dataSource] = player
                    Log.e("SgVideoPlayerPlugin", "new " + handle.id() + "-->" + call.method)
                }

                player?.setEventChannel(eventChannel)
                player?.setSurface(handle)
                val reply = HashMap<String, Any?>()
                reply["textureId"] = handle.id()

                var info = player?.getInitializedInfo()
                reply["info"] = info
                result.success(reply)


                /*if (call.argument<Any>("asset") != null) {
                    val assetLookupKey: String
                    if (call.argument<Any>("package") != null) {
                        assetLookupKey = registrar.lookupKeyForAsset(call.argument("asset"), call.argument("package"))
                    } else {
                        assetLookupKey = registrar.lookupKeyForAsset(call.argument("asset"))
                    }
                    player = VideoPlayer(
                            registrar.context(),
                            eventChannel,
                            handle,
                            "asset:///$assetLookupKey",
                            result)
                    videoPlayers.put(handle.id(), player)
                } else {
                    player = VideoPlayer(
                            registrar.context(), eventChannel, handle, call.argument("uri"), result)
                    videoPlayers.put(handle.id(), player)
                }*/
            }
            else -> {
                val textureId = (call.argument<Any>("textureId") as Number).toLong()
                val dataSource = call.argument<Any>("uri") as String
                val player = cachePlayers[dataSource]
                if (player == null) {
                    result.error(
                            "Unknown textureId",
                            "No video player associated with texture id $textureId", null)
                    return
                }
                onMethodCall(call, result, player)
                Log.i("SgVideoPlayerPlugin", "" + textureId + "-->" + call.method)
            }
        }
    }

    private fun onMethodCall(call: MethodCall, result: Result, player: VideoPlayer?) {
        when (call.method) {
            "setLooping" -> {
                player!!.setLooping(call.argument<Boolean>("looping")!!)
                result.success(null)
            }
            "setVolume" -> {
                player!!.setVolume(call.argument<Double>("volume")!!)
                result.success(null)
            }
            "play" -> {
                player!!.play()
                result.success(null)
            }
            "pause" -> {
                player!!.pause()
                result.success(null)
            }
            "seekTo" -> {
                val location = (call.argument<Any>("location") as Number).toInt()
                player!!.seekTo(location)
                result.success(null)
            }
            "position" -> {
                result.success(player!!.position)
                player.sendBufferingUpdate()
            }
            "dispose" -> {
                player!!.dispose()
                result.success(null)
            }
            "debug" -> {
                val debugCode = (call.argument<Any>("debugCode") as Number).toInt()
                player!!.debug(debugCode)
                result.success(null)
            }
            else -> result.notImplemented()
        }
    }
}
