package com.sogou.shortvideo.videolist

import android.content.Context
import android.content.Intent
import android.graphics.Point
import android.os.Build
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.support.v7.widget.RecyclerView
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.WindowManager
import android.widget.ImageView
import android.widget.RelativeLayout
import cn.iwgang.familiarrecyclerview.FamiliarRecyclerView
import cn.iwgang.familiarrecyclerview.FamiliarRecyclerViewOnScrollListener
import com.bumptech.glide.Glide
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import com.sogou.shortvideo.MyApp
import com.sogou.shortvideo.R
import com.sogou.shortvideo.VideoPageActivity
import com.sogou.shortvideo.net.RetrofitEngine
import com.sogou.shortvideo.net.VideoJson
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import kotlin.math.roundToInt

class VideoListActivity : AppCompatActivity() {

    private lateinit var adapter: MyAdapter
    private lateinit var recyclerView: FamiliarRecyclerView

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_video_list)

        recyclerView = findViewById(R.id.recycler_view)

        adapter = MyAdapter(this)

        recyclerView.setOnItemClickListener { familiarRecyclerView, view, position ->

            var videoBean = adapter.getItem(position) as VideoBean
            var it = Intent(VideoListActivity@this, VideoPageActivity::class.java)
//            var it = Intent(VideoListActivity@this, VideoPageFlutterActivity::class.java)
            it.putExtra("video", videoBean)
            startActivity(it)

        }

        recyclerView.setOnScrollListener(object : FamiliarRecyclerViewOnScrollListener(recyclerView.layoutManager){

            override fun onScrolledToTop() {
//                fetchData(true)
            }

            override fun onScrolledToBottom() {
                fetchLocal(false)
            }

        })

        recyclerView.adapter = adapter
        
//        fetchData(true)
        fetchLocal(true)


    }


    fun fetchLocal(refresh: Boolean){
        Thread(Runnable {
            val type = object :TypeToken<Result<MutableList<VideoBean>>>(){}.type
            var list: Result<MutableList<VideoBean>> = Gson().fromJson<Result<MutableList<VideoBean>>>(VideoJson.json, type)
            runOnUiThread {
                if(refresh){
                    adapter.setData(list.list)
                }else{
                    adapter.appendData(list.list)
                }
            }
        }).start()
    }

    fun fetchData(refresh: Boolean){
        val url = "https://bazinga.mse.sogou.com/channel?h=ffffffff-9326-58e8-4933-e87c0033c587&r=0000&v=5.23.0&hv=COR-AL00&pv=ANDROID9&wv=3.1.0.3049&oaid=dfdebaee-b7ff-f1fc-be77-fdfa6f7df6b0&newType=1&city=%E5%8C%97%E4%BA%AC&nt=wifi&latitude=39.993671896250234&channel=%E5%B0%8F%E8%A7%86%E9%A2%91&mid=e87d91d3c322893c&mode=up&t=1569403548&refresh_cold_start=6&imei=869740032381122&refresh_today=12&api=1&machineType=COR-AL00&longitude=116.33215545079442"
        val call = RetrofitEngine.create(VideoService::class.java).fetchVideoList(url)
        call.enqueue(object : Callback<Result<MutableList<VideoBean>>>{

            override fun onResponse(call: Call<Result<MutableList<VideoBean>>>, response: Response<Result<MutableList<VideoBean>>>) {
                Log.e("onResponse", response.body()!!.list.size.toString())
                if (refresh){
                    adapter.setData(response.body()!!.list)
                }else{
                    adapter.appendData(response.body()!!.list)
                }

            }

            override fun onFailure(call: Call<Result<MutableList<VideoBean>>>, t: Throwable) {
                Log.e("onResponse", t.message)
            }

        })
    }

    class MyAdapter : RecyclerView.Adapter<MyViewHolder> {

        var list: MutableList<VideoBean> = ArrayList()
        
        val width: Int
        val height: Int
        
        constructor(ctx: Context): super(){
            var screen = getScreenRealWidth()
            var divider = ctx.resources.getDimension(R.dimen.dimen_for_divider)
            width = ((screen - divider)/2).roundToInt()
            height = width * 25 / 19
        }

        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): MyViewHolder {
            var view = LayoutInflater.from(parent!!.context).inflate(R.layout.video_item, parent, false)
            return MyViewHolder(view)
        }

        override fun onBindViewHolder(holder: MyViewHolder, position: Int) {

            var video = list.get(position)
            var view = holder.img
            changeSize(view, width, height)
            Glide.with(view).load(video.picurls[0]).into(view)
        }

        override fun getItemCount(): Int {
            return list.size
        }

        fun getItem(position: Int): Any {
            return list.get(position)
        }

        fun setData(list: MutableList<VideoBean>) {
            this.list = list
            notifyDataSetChanged()
        }

        fun appendData(list: MutableList<VideoBean>) {
            this.list.addAll(list)
            notifyItemRangeChanged(itemCount - list.size, itemCount)
//            notifyDataSetChanged()
        }

        private fun changeSize(v: View, width: Int, height: Int) {
            val lp = v.layoutParams as RelativeLayout.LayoutParams
            if (lp.width != width) {
                lp.width = width
                lp.height = height
            }
        }

        fun getScreenRealWidth(): Int {
            val width: Int
            val display = (MyApp.instance!!.getSystemService(Context.WINDOW_SERVICE) as WindowManager).defaultDisplay
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR1) {
                val point = Point()
                display.getRealSize(point)
                width = point.x
            } else {
                width = display.width
            }
            return width
        }

    }

    class MyViewHolder : RecyclerView.ViewHolder {

        var img: ImageView

        constructor(itemView: View) : super(itemView){

            img = itemView.findViewById(R.id.img)
        }
    }
}


