package com.sogou.shortvideo

import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.PluginRegistry

class VideoPlayerPlugin : MethodChannel.MethodCallHandler {

    private var registrar: PluginRegistry.Registrar

    private var videoPluginImpl : IVideoPlugin

    constructor(registrar: PluginRegistry.Registrar, plugin: IVideoPlugin){
        this.registrar = registrar
        this.videoPluginImpl = plugin
    }

    companion object{
        
        private const val PLUGIN_VIDEO_PLAYER: String = "sogou.flutter/videoPlayer"
        
        fun register(pluginRegistry : PluginRegistry, plugin: IVideoPlugin){
            registerWith(pluginRegistry.registrarFor(PLUGIN_VIDEO_PLAYER), plugin)
        }

        fun registerWith(registrar: PluginRegistry.Registrar, plugin: IVideoPlugin){
            var channel = MethodChannel(registrar.messenger(), PLUGIN_VIDEO_PLAYER)
            var instance = VideoPlayerPlugin(registrar, plugin)
            instance.onCreate(registrar)
            channel.setMethodCallHandler(instance)
            registrar.addViewDestroyListener {
                instance.onDestroy()
                false // We are not interested in assuming ownership of the NativeView.
            }
        }
    }

    private fun onCreate(registrar: PluginRegistry.Registrar) {
        if (videoPluginImpl != null){
            videoPluginImpl.onCreate(registrar)
        }
    }
    
    private fun onDestroy() {
        if (videoPluginImpl != null){
            videoPluginImpl.onDestroy()
        }
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        val textures = registrar.textures()
        if (textures == null) {
            result.error("no_activity", "video_player plugin requires a foreground activity", null)
            return
        }

        if (videoPluginImpl != null){
            videoPluginImpl.dispatchCall(registrar, call, result)
        }
    }
}