import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:short_video/network/root_url.dart';
import 'package:short_video/util/validator.dart';
import 'package:short_video/widget/common/base_navi_screen.dart';
import 'package:short_video/widget/common/common_expansion_item.dart';

/// debug 设置 网络相关
///
class DebugPanelNetwork extends CommonExpansionItem {
  bool _isExpaneded;

  final _keyForm = GlobalKey<FormState>();

  final _clTextValue = TextEditingController();

  @override
  Widget buildExpandedBody(BuildContext context) {
    return Form(
      key: _keyForm,
//      padding: EdgeInsets.all(5),
      child: Column(
        children: <Widget>[
          Divider(),
          ListTile(
            title: Text("Business Root:"),
            subtitle: Text(UrlRoot.root(RootType.BUSINESS)),
            onTap: () {
              _clTextValue.value =
                  TextEditingValue(text: UrlRoot.root(RootType.BUSINESS));
              showDialog<String>(
                context: context,
                builder: (context) {
                  return buildCommonInputDlgContent(
                      context, RootType.BUSINESS);
                },
              ).then<void>((String value) {
                // The value passed to Navigator.pop() or null.
                if (isURL(value)) {
                  UrlRoot.resetRoot(value, RootType.BUSINESS);
                  _keyForm.currentState.setState(() {});
                } else {
                  BaseStateLessNaviScreen.currentNaviScreenKey.currentState
                      .showSnackBar(
                          SnackBar(content: Text('This is not valid url')));
                }
              });
            },
          ),
          Divider(),
          ListTile(
            title: Text("Auth Root:"),
            subtitle: Text(UrlRoot.root(RootType.ACCESS_TOKEN)),
            onTap: () {
              _clTextValue.value =
                  TextEditingValue(text: UrlRoot.root(RootType.ACCESS_TOKEN));
              showDialog<String>(
                context: context,
                builder: (context) {
                  return buildCommonInputDlgContent(context, RootType.ACCESS_TOKEN);
                },
              ).then<void>((String value) {
                // The value passed to Navigator.pop() or null.
                if (isURL(value)) {
                  UrlRoot.resetRoot(value, RootType.ACCESS_TOKEN);
                } else {
                  BaseStateLessNaviScreen.currentNaviScreenKey.currentState
                      .showSnackBar(
                          SnackBar(content: Text('This is not valid url')));
                }
              });
            },
          ),
          Divider(),
          ListTile(
            title: Text("Upload Root:"),
            subtitle: Text(UrlRoot.root(RootType.UPDATE)),
            onTap: () {
              _clTextValue.value =
                  TextEditingValue(text: UrlRoot.root(RootType.UPDATE));
              showDialog<String>(
                context: context,
                builder: (context) {
                  return buildCommonInputDlgContent(context, RootType.UPDATE);
                },
              ).then<void>((String value) {
                // The value passed to Navigator.pop() or null.
                if (isURL(value)) {
                  UrlRoot.resetRoot(value, RootType.UPDATE);
                } else {
                  BaseStateLessNaviScreen.currentNaviScreenKey.currentState
                      .showSnackBar(
                          SnackBar(content: Text('This is not valid url')));
                }
              });
            },
          ),
          Divider(),
          ListTile(
            title: Text("Operation Root:"),
            subtitle: Text(UrlRoot.root(RootType.OPERATION)),
            onTap: () {
              _clTextValue.value =
                  TextEditingValue(text: UrlRoot.root(RootType.OPERATION));
              showDialog<String>(
                context: context,
                builder: (context) {
                  return buildCommonInputDlgContent(
                      context, RootType.OPERATION);
                },
              ).then<void>((String value) {
                // The value passed to Navigator.pop() or null.
                if (isURL(value)) {
                  UrlRoot.resetRoot(value, RootType.OPERATION);
                } else {
                  BaseStateLessNaviScreen.currentNaviScreenKey.currentState
                      .showSnackBar(
                          SnackBar(content: Text('This is not valid url')));
                }
              });
            },
          ),
          const SizedBox(height: 24.0),
        ],
      ),
    );
  }

  @override
  String getHeaderName() {
    // TODO: implement getHeaderName
    return "Network";
  }

  @override
  String getHeaderHint() {
    // TODO: implement getHaderHint
    return "root url";
  }

  @override
  String getHeaderValue() {
    // TODO: implement getHaderValue
    return "reset root url";
  }

  /// 一直打开
  @override
  bool isExpanded() {
    if (_isExpaneded == null) {
      return false;
    }
    return _isExpaneded;
  }

  @override
  void resetExpanded(bool expaned) {
    // TODO: implement resetExpaned
    _isExpaneded = expaned;
  }

  @override
  void dispose() {
    // TODO: implement dispose
  }

  @override
  void initState() {
    // TODO: implement initState
  }

  Widget buildSelectDlgContent(BuildContext context, List<String> itemList) {
    var widgetList = itemList.map((value) {
      return SimpleDialogOption(
        onPressed: () {
          Navigator.pop(context, value);
        },
        child: Row(
          mainAxisAlignment: MainAxisAlignment.start,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            Text(value),
          ],
        ),
      );
    }).toList();
    widgetList.add(SimpleDialogOption(
      onPressed: () {},
      child: Row(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          Text("Add New One'"),
        ],
      ),
    ));

    return SimpleDialog(
      title: const Text('Select Host'),
      children: widgetList,
    );
  }

  Widget buildCommonInputDlgContent(BuildContext context, RootType rootType) {
    return AlertDialog(
      title: Text('$rootType'),
      content: Container(
        constraints: BoxConstraints(minWidth: double.maxFinite, maxWidth: double.maxFinite, minHeight: 0),
        child:
          TextField(
            controller: _clTextValue,
//              initialValue: UrlRoot.getProxyPort().toString(),
//          inputFormatters: [WhitelistingTextInputFormatter.digitsOnly],
            decoration: const InputDecoration(
              border: UnderlineInputBorder(),
              filled: true,
//          icon: Icon(Icons.contacts),
              hintText: 'Please input something',
//          labelText: 'Port *',
            ),
//              onSaved: (String value) {
//                UrlRoot.resetProxyPort(int.parse(value));
//              },
//              validator: _validatePort,
          ),
      ),
      actions: <Widget>[
        FlatButton(
            child: const Text('Cancle'),
            onPressed: () {
              Navigator.pop(context);
            }),
        FlatButton(
            child: const Text('OK'),
            onPressed: () {
              Navigator.pop(context, _clTextValue.value.text);
            })
      ],
    );
  }
}
