import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:short_video/base/i_log.dart';
import 'package:short_video/network/root_url.dart';
import 'package:short_video/widget/common/common_expansion_item.dart';

/// debug 设置 网络代理相关
///

class DebugPanelProxy extends CommonExpansionItem {
  FocusNode _fnHost = FocusNode();
  FocusNode _fnPort = FocusNode();

  final _clHost = TextEditingController();
  final _clPort = TextEditingController();

  @override
  Widget buildExpandedBody(BuildContext context) {
    ILog.lg.i("buildExpandedBody invoked");
    return Form(child: Builder(builder: (BuildContext context) {
      return Padding(
        padding: EdgeInsets.only(left: 15, right: 15),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: <Widget>[
//          const SizedBox(height: 24.0),
            TextField(
//              initialValue: UrlRoot.getProxyHost(),
              controller: _clHost,
              focusNode: _fnHost,
              textCapitalization: TextCapitalization.words,
              decoration: const InputDecoration(
                border: UnderlineInputBorder(),
                filled: true,
                icon: Icon(Icons.settings_ethernet),
                hintText: 'Proxy IP required',
                labelText: 'Host *',
              ),
//              onSaved: (String value) {
//                UrlRoot.resetProxyHost(value);
//              },
//              validator: _validateHost,
            ),
            const SizedBox(height: 24.0),
            TextField(
              focusNode: _fnPort,
              controller: _clPort,
//              initialValue: UrlRoot.getProxyPort().toString(),
              inputFormatters: [WhitelistingTextInputFormatter.digitsOnly],
              decoration: const InputDecoration(
                border: UnderlineInputBorder(),
                filled: true,
                icon: Icon(Icons.contacts),
                hintText: 'Proxy Port required',
                labelText: 'Port *',
              ),
//              onSaved: (String value) {
//                UrlRoot.resetProxyPort(int.parse(value));
//              },
//              validator: _validatePort,
            ),
            const SizedBox(height: 24.0),
          ],
        ),
      );
    }));
  }

  bool _validateHost(String value) {
    if (value.isEmpty) return false;
    final RegExp nameExp = RegExp(r'^(?:[0-9]{1,3}.){3}[0-9]{1,3}$');
    if (!nameExp.hasMatch(value)) return false;
    return true;
  }

  bool _validatePort(String value) {
    if (value.isEmpty) return false;
    final RegExp nameExp = RegExp(r'^[1-9]\d*$');
    if (!nameExp.hasMatch(value)) return false;
    return true;
  }

  @override
  String getHeaderName() {
    return "Proxy";
  }

  @override
  String getHeaderHint() {
    return "proxy now enabled";
  }

  @override
  String getHeaderValue() {
    return "proxy disabled";
  }

  @override
  bool isExpanded() {
    return UrlRoot.proxyEnable();
  }

  @override
  void resetExpanded(bool expaned) {
    UrlRoot.resetProxyEnable(expaned);
  }

  @override
  void dispose() {
    // Clean up the focus node when the Form is disposed
    _fnHost.dispose();
    _fnPort.dispose();
  }

  @override
  void initState() {
    _fnHost.addListener(() {
      if (!_fnHost.hasFocus) {
        if (_validateHost(_clHost.value.text)) {
          UrlRoot.resetProxyHost(_clHost.value.text);
//          Fluttertoast.showToast(
//            msg: "Proxy Host setted",
//            toastLength: Toast.LENGTH_SHORT,
//            gravity: ToastGravity.BOTTOM,
//            timeInSecForIos: 1,
//        bgcolor: "#e74c3c",
//        textcolor: '#ffffff'
//          );
        } else {
//          Fluttertoast.showToast(
//            msg: "Proxy Host is not valid",
//            toastLength: Toast.LENGTH_SHORT,
//            gravity: ToastGravity.BOTTOM,
//            timeInSecForIos: 1,
//        bgcolor: "#e74c3c",
//        textcolor: '#ffffff'
//          );
        }
      }
    });
    _fnPort.addListener(() {
      if (!_fnPort.hasFocus) {
        if (_validatePort(_clPort.value.text)) {
          int portNew = int.parse(_clPort.value.text);
          UrlRoot.resetProxyPort(portNew);
//          Fluttertoast.showToast(
//            msg: "Proxy Port setted",
//            toastLength: Toast.LENGTH_SHORT,
//            gravity: ToastGravity.BOTTOM,
//            timeInSecForIos: 1,
////        bgcolor: "#e74c3c",
////        textcolor: '#ffffff'
//          );
        } else {
//          Fluttertoast.showToast(
//            msg: "Proxy Port is not valid",
//            toastLength: Toast.LENGTH_SHORT,
//            gravity: ToastGravity.BOTTOM,
//            timeInSecForIos: 1,
////        bgcolor: "#e74c3c",
////        textcolor: '#ffffff'
//          );
        }
      }
    });

    String defaultValue = UrlRoot.getProxyHost();
    _clHost.value = TextEditingValue(
        text: defaultValue,
        selection: TextSelection.collapsed(offset: defaultValue.length));

    defaultValue = UrlRoot.getProxyPort().toString();
    _clPort.value = TextEditingValue(
        text: defaultValue,
        selection: TextSelection.collapsed(offset: defaultValue.length));
  }
}
