import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:short_video/base/i_log.dart';
import 'package:short_video/debug/debug_s_common_action.dart';
import 'package:short_video/debug/debug_s_network.dart';
import 'package:short_video/debug/debug_s_proxy.dart';
import 'package:short_video/widget/common/base_navi_screen.dart';
import 'package:short_video/widget/common/common_expansion_item.dart';


class DebugScreen extends BaseStateLessNaviScreen {

  @override
  Widget body(BuildContext context) => DebugContent();

  @override
  String title(BuildContext context) {
    return "DebugScreen title";
  }

  @override
  Future<bool> onBackPressed(BuildContext context) {
    Navigator.pop(context);
    return Future.value(false);
  }
}

class DebugContent extends StatefulWidget {
  static final  GlobalKey<_DebugContentState> debugScreenKey = GlobalKey<_DebugContentState>();
  DebugContent() : super(key: debugScreenKey);

  @override
  State<StatefulWidget> createState() {
    return _DebugContentState();
  }
}

class _DebugContentState extends State<DebugContent> {
  Widget contentWidget;
  List<CommonExpansionItem> _demoItems;

  @override
  void initState() {
    super.initState();
    _demoItems = [
      DebugPanelProxy(),
      DebugPanelNetwork(),
      DebugPanelCommon(),
    ];

    _demoItems.forEach((item) {
      item.initState();
    });
  }

  @override
  void dispose() {
    _demoItems.forEach((item) {
      item.dispose();
    });

    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    ILog.lg.i("_DebugContentState::build invoked");
    contentWidget = new SingleChildScrollView(
      child: SafeArea(
          child: Container(
        child: ExpansionPanelList(
            expansionCallback: (int index, bool isExpanded) {
              setState(() {
                _demoItems[index].resetExpanded(!isExpanded);
              });
            },
            children:
                _demoItems.map<ExpansionPanel>((CommonExpansionItem item) {
              return ExpansionPanel(
                  isExpanded: item.isExpanded(),
                  headerBuilder: item.headerBuilder,
                  body: item.buildExpandedBody(context));
            }).toList()),
        margin: const EdgeInsets.all(24.0),
      )),
    );
    return contentWidget;
  }
}
