// Copyright 2018 The Flutter Architecture Sample Authors. All rights reserved.
// Use of this source code is governed by the MIT license that can be found
// in the LICENSE file.


import 'dart:async';

import 'package:flutter/widgets.dart';
import 'package:intl/intl.dart';
import 'package:short_video/locale/messages_all.dart';

class LoanLocalizations {

  LoanLocalizations(this.locale);

  final Locale locale;

  static Future<LoanLocalizations> load(Locale locale) {
    return initializeMessages(locale.toString()).then((_) {
      return LoanLocalizations(locale);
    });
  }

  static LoanLocalizations of(BuildContext context) {
    return Localizations.of<LoanLocalizations>(
        context, LoanLocalizations);
  }

  String get appName => Intl.message(
    '搜狗浏览器',
    name: 'appName',
    args: [],
    locale: locale.toString(),
  );
}

class LoanLocalizationsDelegate
    extends LocalizationsDelegate<LoanLocalizations> {
  @override
  Future<LoanLocalizations> load(Locale locale) =>
      Future(() => LoanLocalizations(locale));

  @override
  bool shouldReload(LoanLocalizationsDelegate old) => false;

  @override
  bool isSupported(Locale locale) =>
      ['en', 'zh'].contains(locale.languageCode);
}
