import 'dart:async';

import 'package:connectivity/connectivity.dart';
import 'package:flutter/material.dart';
import 'package:scoped_model/scoped_model.dart';
import 'package:short_video/base/i_log.dart';
import 'package:short_video/bean/support_bean.dart';
import 'package:short_video/bean/video_item.dart';
import 'package:short_video/platform/sg_operation.dart';
import 'package:short_video/platform/sg_player.dart';
import 'package:short_video/player_pool.dart';
import 'package:short_video/util/constant.dart';
import 'package:short_video/util/statistics_helper.dart';
import 'package:short_video/widget/ad_text_widget.dart';
import 'package:short_video/widget/givelike_anim_screen.dart';
import 'dart:core';

import 'bean/heart_info.dart';
import 'main_screen.dart';

typedef PlayCompleteCallback = dynamic Function(int index);

/// 处理单个视频相关
class VideoItemModel extends Model {
  final VideoItem _videoItem;
  final int _index;
  final PlayCompleteCallback _onPlayComplete;
  InheritedWidget aa;
  VideoPlayerController _videoPlayerController;
  bool _isError = false;
  bool _isNoNet = false;
  bool _selected = false;

  // 由于处理多个controller，旧的视屏被重新选中后，需要用此标志位，而不能仅仅用controller的标志位
  bool _pauseVisible = false;

  // 统计用；计算视频播放时间
  DateTime _videoStartTime;

  // 上次评论的内容；
  String _commentText;
  bool _isLooping = true;
  bool _isAdShowed = false; //广告是否展示过

  // 用户点击评论后，评论数
  int _currCommentCount;

  // 是否已经收藏过
  bool _new_is_support;

  // 收藏数
  int _currSupportCount;

  // 忽略 native透传的pending状态
  bool _ignorePending = false;

  ListHeartValeNotifier _heartAnimNotifier =
      ListHeartValeNotifier(List<HeartInfo>());

  ValueNotifier<SupportBean> _notiSupportInfo;

  VideoItemModel(this._videoItem, this._index, this._onPlayComplete) {
    _notiSupportInfo =
        ValueNotifier<SupportBean>(SupportBean(isSupported, currSupportCount));
  }

  ListHeartValeNotifier get heartAnimNotifier => _heartAnimNotifier;

  ValueNotifier<SupportBean> get notiSupportInfo => _notiSupportInfo;

  int get index => _index;

  int get currCommentCount {
    if (_currCommentCount == null) {
      _currCommentCount = _videoItem.comment_cnt;
    }
    return _currCommentCount;
  }

  DateTime get videoStartTime {
    return _videoStartTime;
  }

  set currCommentCount(int newValue) {
    _currCommentCount = newValue;
    notifyListeners();
  }

  int get currSupportCount {
    if (_currSupportCount == null) {
      _currSupportCount = _videoItem.support_num;
    }
    return _currSupportCount;
  }

  set currSupportCount(int newValue) {
    _currSupportCount = newValue;
    notifyListeners();
  }

  bool get isSupported {
    if (_new_is_support == null) {
      _new_is_support = _videoItem.is_support;
    }
    return _new_is_support;
  }

  set isSupported(bool newValue) {
    _new_is_support = newValue;
    notifyListeners();
  }
  /// 只有在被选中的时候，才显示错误、没有网络等状态
  bool get isError{
    if(_selected){
      return _isError;
    }else{
      return false;
    }
  }

  /// 是否有网络
  bool get isNoNet{
    if(_selected){
      return _isNoNet;
    }else{
      return false;
    }
  }

  bool _showPending = true;

  bool get isLoading {
    return _showPending;
//      if (_videoPlayerController == null) {
//        return true;
//      }
//      return _videoPlayerController.value.pending;

//    if(_selected){
//      return _showPending;
//
////      if (_videoPlayerController == null) {
////        return true;
////      }
////      return _videoPlayerController.value.pending;
//    }else{
//      return _isError || _isNoNet;
//    }
  }

  bool get isAdShowed => _isAdShowed;

  bool get showPlayer {
    if (_videoPlayerController == null) {
      return false;
    } else {
      if (_videoPlayerController.value.status != PlayerStatus.error &&
          _videoPlayerController.value.status != PlayerStatus.unknown &&
          _videoPlayerController.value.status != PlayerStatus.created) {
        return true;
      } else {
        return false;
      }
    }
  }

  set isLooping(bool newLoop) {
    _isLooping = newLoop;
    if (_videoPlayerController != null) {
      _videoPlayerController.setLooping(_isLooping);
    }
  }

  String get commentText => _commentText;

  bool get pauseVisible => _pauseVisible;

  /// 设置单击之后的执行的操作
  void setPauseOrResume() {
    if (_videoPlayerController == null ||
        (_videoPlayerController.value.status == PlayerStatus.unknown)) {
      return;
    }
    if (_videoPlayerController.value.status == PlayerStatus.playing) {
      _pauseVisible = true;
      applyPause();
    } else {
      _pauseVisible = false;
      applyPlay();
    }
//    notifyListeners();
  }

  void applyPause([needNofiyListener = true]) {
    if (_videoPlayerController != null) {
//      ILog.lg.i("aaa-applyPause:$_index");
      _videoPlayerController.pause();
      SgOperation.stayScreenLight(false);
      // 发送pingback
      _tryToPostPingbackPlayDuring();
      if (needNofiyListener) {
        notifyListeners();
      }
    }
  }

  void applyPlay([needNofiyListener = true]) {
    if (_videoPlayerController != null) {
      ILog.lg.i("bbb-applyPlay:$_index");
    // 开始监听pending状态
      _ignorePending = false;
      _videoPlayerController.play();
      SgOperation.stayScreenLight(true);
      // 启动计时
      _videoStartTime = DateTime.now();
      if (needNofiyListener) {
        notifyListeners();
      }
    }
  }

  /// native执行了评论操作，评论数 + 1；
  void onNativeCommentResult(String oldCommentStr, int newCommentCount) {
    _commentText = oldCommentStr;
    _currCommentCount = newCommentCount;
    notifyListeners();
  }

  VideoItem get videoItem => _videoItem;

  VideoPlayerController get playerController => _videoPlayerController;

  void onClickErrorRetry() async {
    // 清理标志位
    _pauseVisible = false;
    _isError = false;

    ConnectivityResult result = await Connectivity().checkConnectivity();
    if (result == ConnectivityResult.none) {
      _isNoNet = true;
      _isError = true;
    } else {
      _isNoNet = false;
    }

    if (_isError) {
      notifyListeners();
      return;
    }

    if (_videoPlayerController != null) {
      _videoPlayerController.resetDataSource(_videoItem.video_url);
      notifyListeners();
      return;
    }

    StatisticsHelper.shortVideoPlayer(_videoItem);
    if (_videoItem.category != null &&
        (_videoItem.adtype == 1 || _videoItem.adtype == 0)) {
      //发送广告展现pingback和trackurl
      if (_isAdShowed == false) {
        _isAdShowed = true;
        StatisticsHelper.sendAdShowPingback(_videoItem);
      }
    }

    notifyListeners();
  }

  /// 点击 收藏 按钮
  /// 注：不必notifyListeners，因为givelike button会自动加一，根据返回值；
  Future<bool> clickLikeVideoButtonWidget(bool isLiked) async {
    bool actRe = await _doGiveLikeAction();
    if (actRe) {
      // 通知开始动画
      double dx = Adapt.screenW() / 2.0;
      double dy = Adapt.screenH() / 2.0;
      HeartInfo info =
          HeartInfo(DateTime.now().millisecondsSinceEpoch, dx, dy, 0);
      _heartAnimNotifier.changeVideoHeartInfoNumber(info);
      // _notiSupportInfo 不发通知，否则点赞动画无法执行
    }

    return isSupported;
  }

  // 双击屏幕点赞 ， 此需要更新
  void onDoubleClickGiveLike(HeartInfo heart) async {
    await _doGiveLikeAction();
    // 开始动画
    _heartAnimNotifier.changeVideoHeartInfoNumber(heart);
    // 更新，通知点赞按钮重绘
    _notiSupportInfo.value = SupportBean(isSupported, currSupportCount);
  }

  Future<bool> _doGiveLikeAction() async {
    if (_videoItem.category != null ||
        (_new_is_support != null && _new_is_support)) {
      //广告不允许点赞 || 已经点过了
      return Future(() => false);
    }

    // 执行native
    Map param = new Map();
    param['newsid'] = _videoItem.newsid ?? "";
    param['url'] = _videoItem.url ?? "";
    param['title'] = _videoItem.title ?? "";
    int nativeRes = await SgOperation.giveLike(param);
    ILog.lg.i("SgOperation.giveLike:$nativeRes");
    if (nativeRes == null || nativeRes != 1) {
      //native 不成功
      return false;
    }

    // 变更model状态
    _new_is_support = true;

    if (_currSupportCount == null) {
      _currSupportCount = _videoItem.support_num;
    }

    if (_currSupportCount == null) {
      _currSupportCount = 1;
    } else {
      _currSupportCount++;
    }

    //pingback
    String pingbackKey = "ShortVideoLikeClick";
    Map pingbackParam = new Map();
    pingbackParam["name"] = Constant.CHANNEL ?? "";
    pingbackParam["url"] = _videoItem.url ?? "";
    pingbackParam["title"] = _videoItem.title ?? "";
    pingbackParam["newsid"] = _videoItem.newsid ?? "";
    Map dataMap = new Map();
    dataMap["key"] = pingbackKey;
    dataMap["param"] = pingbackParam;
    SgOperation.statistics(SgOperation.STATISTICS_IMMEDIATELY_TYPE, dataMap);

    return true;
  }

  void onPageChanged(int currPageIndex) {
    if (currPageIndex == _index) {
      // 被选中
      _selected = true;
      _doSelected();
    } else if ((currPageIndex - _index).abs() == 1) {
      // 下一个
      _selected = false;
      _doPrefetch();
    } else if ((currPageIndex - _index).abs() == 2) {
      _selected = false;
      // 下两个
      _doClear();
    } else {
      _selected = false;
      // nothing to do
    }
  }

  void _doSelected() async {
    // 清理标志位
    _isError = false;
    _isNoNet = false;
    _pauseVisible = false;
    if (_videoPlayerController == null) {
      // 清理预取
      if (_tPrefetch != null) {
        _tPrefetch.cancel();
        _tPrefetch = null;
      }
      _setupPlayer();
    } else {
      // 进入时，有可能有错误发生
      if (_videoPlayerController.value.status == PlayerStatus.error) {
        // 出错后，重新加载
        onClickErrorRetry();
      } else if (_videoPlayerController.value.status ==
              PlayerStatus.initialized ||
          _videoPlayerController.value.status == PlayerStatus.paused ||
          _videoPlayerController.value.status == PlayerStatus.completed) {
        // 有可能被选中的那一刻，屏幕被遮挡了
        if(!MainScreen.mainScreenKey.currentState.isHiden){
          applyPlay(false);
        }
      } else {
        //  _setupPlayer 已经设置完毕回调。初始化完毕之后，会自动播放；
        // 故，此处不执行play操作
      }
    }

    StatisticsHelper.shortVideoPlayer(_videoItem);
    if (_videoItem.category != null &&
        (_videoItem.adtype == 1 || _videoItem.adtype == 0)) {
      //发送广告展现pingback和trackurl
      if (_isAdShowed == false) {
        _isAdShowed = true;
        StatisticsHelper.sendAdShowPingback(_videoItem);
      }
    }

    notifyListeners();
  }

  Timer _tPrefetch;

  void _doPrefetch() {
    _commentText = null;

    if (_videoPlayerController != null) {
      // 将播放进度放到开始，防止收到complete事件
      if (_videoPlayerController.value.status == PlayerStatus.playing ||
          _videoPlayerController.value.status == PlayerStatus.paused ||
          _videoPlayerController.value.status == PlayerStatus.completed) {
        _ignorePending = true;
        _videoPlayerController.seekTo(Duration.zero);
      }
      applyPause(false);
      // 此处需要 _isPaused 改为 false，避免滑动回来的时候，显示暂停按钮
      _pauseVisible = false;
    } else if (_tPrefetch == null) {
      // 预取
      _tPrefetch = Timer(Duration(milliseconds: 200), () {
        _setupPlayer();
        _tPrefetch = null;
      });
    } else {
      // 已经有在预取的了
    }
  }

  void _doClear() {
    if (_videoPlayerController != null) {
      _videoPlayerController.removeListener(_onPlayerStatusChanged);
      _videoPlayerController.stop();
      _videoPlayerController = null;
    }

    if (_tPrefetch != null) {
      _tPrefetch.cancel();
      _tPrefetch = null;
    }
    _isError = false;
    _isNoNet = false;
    _pauseVisible = false;
  }

  void _setupPlayer() async {
    int poolIndex = _index % PlayerPool.PLAYER_COUNT;
    _videoPlayerController = PlayerPool.cachePlayers[poolIndex];
    if (_videoPlayerController == null) {
      _videoPlayerController = VideoPlayerController();
      await _videoPlayerController.init();
      PlayerPool.cachePlayers[poolIndex] = _videoPlayerController;
    }

    _videoPlayerController.setLooping(_isLooping);
    _videoPlayerController.addListener(_onPlayerStatusChanged);

    _videoPlayerController.resetDataSource(_videoItem.video_url ?? "");
  }

  void _onPlayerStatusChanged() async {
    if (_videoPlayerController == null) {
      // item 已经被disposed
      return;
    }

    if(!_ignorePending){
      _showPending = _videoPlayerController.value.pending;
    }

    ILog.lg.i(
        "bbb- _onPlayerStatusChanged:$_index isHiden:${MainScreen.mainScreenKey.currentState.isHiden} PlayerStatus : ${_videoPlayerController.value}");
    // 并非当前正在播放的
    if (!_selected) {
      return;
    }

    // 页面被遮住了onPause/onResume
    if(MainScreen.mainScreenKey.currentState.isHiden){
      return;
    }

    _pauseVisible = _videoPlayerController.value.status == PlayerStatus.paused;
    switch (_videoPlayerController.value.status) {
      case PlayerStatus.unknown:
      case PlayerStatus.created:
        // 第一次进入播放页，首个player被创建 & 播放完毕'stop'被调用： 需要更新loading状态
        notifyListeners();
        break;
      case PlayerStatus.initialized:
        applyPlay(true);
        break;
      case PlayerStatus.playing:
        // 有可能pending状态发生变更
        notifyListeners();
        break;
      case PlayerStatus.paused:
        // 有可能native暂停了
        notifyListeners();
        break;
      case PlayerStatus.completed:
        // 播放完毕，无需 notifyListeners
        if (_onPlayComplete != null) {
          _onPlayComplete(_index);
        }
        break;
      case PlayerStatus.error:
        _isError = true;
        // 再判断是否网络有错误
        ConnectivityResult result = await Connectivity().checkConnectivity();
        if (result == ConnectivityResult.none) {
          _isNoNet = true;
        } else {
          _isNoNet = false;
        }
        notifyListeners();
        break;
    }
  }

  void _sendPingbackPlayDuring(VideoItem videoItem, int informStayTime) {
    if (informStayTime < 0) {
      informStayTime = 0;
    }
    String pingbackKey = "InformStayTime";
    Map pingbackParam = new Map();
    pingbackParam["StayModule"] = "shortvideolist";
    pingbackParam["Channel"] = Constant.CHANNEL ?? "小视频";
    pingbackParam["page-type"] = "video";
    String urlTemp = videoItem.url ?? "";
    Uri u = Uri.parse(urlTemp);
    String newsType = "other";
    if (u.host != null && u.host.toLowerCase().startsWith("bazinga")) {
      newsType = "bazinga";
    }
    pingbackParam["news-type"] = newsType;
    pingbackParam["newsid"] = videoItem.newsid;
    pingbackParam["InformStayTime"] = informStayTime;
    Map dataMap = new Map();
    dataMap["key"] = pingbackKey;
    dataMap["param"] = pingbackParam;
    SgOperation.statistics(SgOperation.STATISTICS_IMMEDIATELY_TYPE, dataMap);
  }

  void _tryToPostPingbackPlayDuring() {
    if (_videoStartTime != null) {
      if (_videoPlayerController == null) {
        _videoStartTime = null;
        return;
      }
      DateTime dateNow = DateTime.now();
      Duration playingDuring = dateNow.difference(_videoStartTime);
      ILog.lg.i("_sendPingbackPlayDuring playingDuring: $playingDuring");
      _sendPingbackPlayDuring(_videoItem, playingDuring.inSeconds);
      _videoStartTime = null;
    }
  }

  void onPause() {
    if(!_selected){
      return;
    }
    applyPause();
  }

  void onResume() {
    if(!_selected){
      return;
    }
    if (!_pauseVisible) {
      // 只有用户手动点击暂停，才恢复
      applyPlay();
    }
  }

  void dispose() {
    _tryToPostPingbackPlayDuring();

    if (_videoPlayerController != null) {
      // 停止正在播放的视频
      if(_videoPlayerController.value.status == PlayerStatus.playing){
        _videoPlayerController.stop();
      }
      // 无需再dispose，因为controller已经被缓存
      //      _videoPlayerController.dispose();
      _videoPlayerController = null;
    }
    if (_heartAnimNotifier != null) {
//      _heartAnimNotifier.dispose();
      _heartAnimNotifier = null;
    }
    if (_notiSupportInfo != null) {
      // 无需dispose；下拉刷新时，diapose会被先调用；然后更新界面的时候，Widget会unmount，也会调removelistner，彼时，状态就错了
//      _notiSupportInfo.dispose();
      _notiSupportInfo = null;
    }

    if (_tPrefetch != null) {
      _tPrefetch.cancel();
      _tPrefetch = null;
    }
  }

  @protected
  void notifyListeners() {
    ILog.lg.invoke("inde: $_index -> ${_videoItem.video_url}");
    super.notifyListeners();
  }

  static VideoItemModel of(BuildContext ctx) {
    return ScopedModel.of<VideoItemModel>(ctx);
  }
}
