import 'package:short_video/base/i_log.dart';
import 'package:short_video/base/i_store_sp.dart';
import 'package:short_video/build_config.dart';

enum RootType {
  ACCESS_TOKEN, // 登录认证
  BUSINESS, // 业务
  UPDATE, // 升级
  TRACK, // 埋点统计
  OPERATION, // 运营广告
  UPLOAD, // 文件上传
}

/// 这些用Preferce 存储； 业务相关数据，用SecureStoreage存储
/// 添加版本控制
class UrlRoot {
  static const int _PREF_VERSION = 100;

//  static const String _DEFAULT_ROOT_HOST_DEBUG = "https://testpassport.yidianfenqi.com/";
  ///临时测试地址: https://testapi.loan.sogou.com/
  ///测试地址: https://testh5.loan.sogou.com/
  static const String _DEFAULT_ROOT_HOST_DEBUG = "http://10.144.65.239/";
  static const String _DEFAULT_ROOT_HOST_RELEASE =
      "https://bazinga.mse.sogou.com/";

//  static const String _DEFAULT_ROOT_HOST_UPLOAD_DEBUG = "http://testupload.loan.sogou.com/";
  static const String _DEFAULT_ROOT_HOST_UPLOAD_DEBUG =
      "http://api.yidianfenqi.com/";
  static const String _DEFAULT_ROOT_HOST_UPLOAD_RELEASE =
      "http://api.yidianfenqi.com/";

  static const String _DEFAULT_ROOT_HOST_AUTH_DEBUG =
      "http://api.yidianfenqi.com/";
  static const String _DEFAULT_ROOT_HOST_AUTH_RELEASE =
      "http://api.yidianfenqi.com/";

  static const String _PREF_KEY_VERSION = "pref_version";

  static const String _PREF_KEY_HOST_AUTH = "pref_key_host_auth";
  static const String _PREF_KEY_HOST_BUSS = "pref_key_host_buss";
  static const String _PREF_KEY_HOST_UPDATE = "pref_key_host_update";
  static const String _PREF_KEY_HOST_OPERATION = "pref_key_host_operation";
  static const String _PREF_KEY_HOST_TRACK = "pref_key_host_operation";
  static const String _PREF_KEY_HOST_UPLOAD = "pref_key_host_upload";

  static const String _PREF_KEY_ENABLE_PROXY = "pref_key_enable_proxy";

  static const String _PREF_KEY_HOST_PROXY = "pref_key_host_proxy";
  static const String _PREF_KEY_PORT_PROXY = "pref_key_port_proxy";

  static String _sCacheHostBuss;
  static String _sCacheHostAuth;
  static String _sCacheHostUpdate;
  static String _sCacheHostOperation;
  static String _sCacheHostTrack;
  static String _sCacheHostUpload;

  static String _sCacheHostProxy;

  static int _sCachePortProxy;

  static bool _sProxyEnabled;

  /// App 启动既调用；加载本地Url配置
  /// 前置依赖：无
  static Future<bool> onAppLaunch() async {
    ILog.lg.i("LAUNCH LOADING UrlRoot ... ... ...");
    int oldVersion;
    try {
      oldVersion = IStoreSp.sp.getInt(_PREF_KEY_VERSION);
    } catch (e) {
      oldVersion = null;
      ILog.lg.e("get int from pref errro: $e");
    }
    if (oldVersion == null) {
      ILog.lg.d("Can not read pref version, maybe first run");
      oldVersion = _PREF_VERSION;
      IStoreSp.sp.setInt(_PREF_KEY_VERSION, _PREF_VERSION);
    }
    // 本地数据版本检查
    if (oldVersion < _PREF_VERSION) {
      if (!_onLocalStorageUpdate(oldVersion, _PREF_VERSION)) {
        ILog.lg.e(
            "_onLocalStorageUpdate ERRRO; Can not convert old date info new formation ${oldVersion}->${_PREF_VERSION}");
        return false;
      } else {
        var updateVersion =
            await IStoreSp.sp.setInt(_PREF_KEY_VERSION, _PREF_VERSION);
        if (!updateVersion) {
          return false;
        }
      }
    }
    _sCacheHostBuss = IStoreSp.sp.getString(_PREF_KEY_HOST_BUSS);
    _sCacheHostAuth = IStoreSp.sp.getString(_PREF_KEY_HOST_AUTH);
    _sCacheHostUpdate = IStoreSp.sp.getString(_PREF_KEY_HOST_UPDATE);
    _sCacheHostOperation = IStoreSp.sp.getString(_PREF_KEY_HOST_OPERATION);
    _sCacheHostTrack = IStoreSp.sp.getString(_PREF_KEY_HOST_TRACK);
    _sCacheHostUpload = IStoreSp.sp.getString(_PREF_KEY_HOST_UPLOAD);

    _sCacheHostProxy = IStoreSp.sp.getString(_PREF_KEY_HOST_PROXY);
    _sCachePortProxy = IStoreSp.sp.getInt(_PREF_KEY_PORT_PROXY);
    _sProxyEnabled = IStoreSp.sp.getBool(_PREF_KEY_ENABLE_PROXY);

    if (_sCacheHostBuss == null || _sCacheHostBuss.length < 1) {
      if (BuildConfig().isInDebugMode()) {
        _sCacheHostBuss = _DEFAULT_ROOT_HOST_DEBUG;
      } else {
        _sCacheHostBuss = _DEFAULT_ROOT_HOST_RELEASE;
      }
    }

    if (_sCacheHostUpdate == null || _sCacheHostUpdate.length < 1) {
      _sCacheHostUpdate = _sCacheHostBuss;
    }
    if (_sCacheHostOperation == null || _sCacheHostOperation.length < 1) {
      _sCacheHostOperation = _sCacheHostBuss;
    }
    if (_sCacheHostTrack == null || _sCacheHostTrack.length < 1) {
      _sCacheHostTrack = _sCacheHostBuss;
    }

    if (_sCacheHostUpload == null || _sCacheHostUpload.length < 1) {
      if (BuildConfig().isInDebugMode()) {
        _sCacheHostUpload = _DEFAULT_ROOT_HOST_UPLOAD_DEBUG;
      } else {
        _sCacheHostUpload = _DEFAULT_ROOT_HOST_UPLOAD_RELEASE;
      }
    }

    if (_sCacheHostAuth == null || _sCacheHostAuth.length < 1) {
      if (BuildConfig().isInDebugMode()) {
        _sCacheHostAuth = _DEFAULT_ROOT_HOST_AUTH_DEBUG;
      } else {
        _sCacheHostAuth = _DEFAULT_ROOT_HOST_AUTH_RELEASE;
      }
    }

    if (_sProxyEnabled == null) {
      _sProxyEnabled = false;
    }

    // make default
//      _sProxyEnabled = true;

    if (_sCacheHostProxy == null) {
      _sCacheHostProxy = "10.129.41.169";
    }
    if (_sCachePortProxy == null) {
      _sCachePortProxy = 8888;
    }

    return true;
  }

  static bool resetRoot(String host, RootType rootType) {
    switch (rootType) {
      case RootType.BUSINESS:
        _sCacheHostBuss = host;
        IStoreSp.sp.setString(_PREF_KEY_HOST_BUSS, _sCacheHostBuss);
        break;
      case RootType.ACCESS_TOKEN:
        _sCacheHostAuth = host;
        IStoreSp.sp.setString(_PREF_KEY_HOST_AUTH, _sCacheHostAuth);
        break;
      case RootType.UPDATE:
        _sCacheHostUpdate = host;
        IStoreSp.sp.setString(_PREF_KEY_HOST_UPDATE, _sCacheHostUpdate);
        break;
      case RootType.OPERATION:
        _sCacheHostOperation = host;
        IStoreSp.sp.setString(_PREF_KEY_HOST_OPERATION, _sCacheHostOperation);
        break;
      case RootType.TRACK:
        _sCacheHostTrack = host;
        IStoreSp.sp.setString(_PREF_KEY_HOST_TRACK, _sCacheHostTrack);
        break;
      case RootType.UPLOAD:
        _sCacheHostUpload = host;
        IStoreSp.sp.setString(_PREF_KEY_HOST_UPLOAD, _sCacheHostUpload);
        break;
    }
    return true;
  }

  static String root(RootType rootType) {
    String res;
    switch (rootType) {
      case RootType.BUSINESS:
        res = _sCacheHostBuss;
        break;
      case RootType.ACCESS_TOKEN:
        res = _sCacheHostAuth;
        break;
      case RootType.UPDATE:
        res = _sCacheHostUpdate;
        break;
      case RootType.OPERATION:
        res = _sCacheHostOperation;
        break;
      case RootType.TRACK:
        res = _sCacheHostTrack;
        break;
      case RootType.UPLOAD:
        res = _sCacheHostUpload;
        break;
    }

    return res;
  }

  static String getProxyHost() {
    return _sCacheHostProxy;
  }

  static int getProxyPort() {
    if (_sCachePortProxy == null) {
      return 0;
    }
    return _sCachePortProxy;
  }

  static void resetProxyHost(String newProxyHost) {
    assert(newProxyHost != null && newProxyHost.length > 1,
        "new proxy host should not be null");
    _sCacheHostProxy = newProxyHost;

    IStoreSp.sp.setString(_PREF_KEY_HOST_PROXY, _sCacheHostProxy);
  }

  static void resetProxyPort(int newProxyPort) {
    assert(newProxyPort != null && newProxyPort > 0 && newProxyPort < 65536,
        "new proxy port not valid");
    _sCachePortProxy = newProxyPort;

    IStoreSp.sp.setInt(_PREF_KEY_PORT_PROXY, _sCachePortProxy);
  }

  static bool proxyEnable() {
    if (_sProxyEnabled == null) {
      return false;
    }
    return _sProxyEnabled;
  }

  static void resetProxyEnable(bool enable) {
    _sProxyEnabled = enable;
    IStoreSp.sp.setBool(_PREF_KEY_ENABLE_PROXY, _sProxyEnabled);
  }

  ///  当版本被覆盖安装时，此方法会被调用；此时可进行数据格式转换等工作
  ///  此只做数据转移就行，无需重设version；会自动完成此工作
  static bool _onLocalStorageUpdate(int oldVersion, int newVersion) {
    return true;
  }
}
