import 'dart:collection';

import 'package:flutter/services.dart';
import 'package:short_video/base/i_log.dart';
import 'package:short_video/util/statistics_helper.dart';

/// platform 为 flutter提供的API支持
///
///
class SgOperation{

  static const _platformAction = const MethodChannel('com.sogou.flutter.plugin/sg_action');

  static const String ACTION_NAVIGATOR = "action_navigator";
  static const String ACTION_STATISTICS = "action_statistics";
  static const String ACTION_WHEN_CRASH = "action_when_crash";
  static const String ACTION_GETSYSINFO = "action_getsysinfo";
  static const String ACTION_PAGECLOSE = "action_pageClose";//页面关闭
  static const String ACTION_FEEDBACK = "action_feedBack";//反馈
  static const String ACTION_NOTLIKE = "action_notLike";//不感兴趣
  static const String ACTION_FAVORITE = "action_favorite";//收藏
  static const String ACTION_OPENADDETAIL = "action_openAdDetail";//打开广告详情
  static const String ACTION_DOWNLOADADAPP = "action_downloadAdApp";//下载广告APP
  static const String ACTION_GIVELIKE = "action_giveLike";//点赞
  static const String ACTION_SHOWCOMMENTS = "action_showComments";//展示评论
  static const String ACTION_SHARE = "action_share";//分享
  static const String ACTION_WRITECOMMENTS = "action_writeComments";//写评论
  static const String ACTION_SWITCH_VIDEO = "action_switch_video";//视频切换
  static const String ACTION_EDUCATION_SHOW = "action_education_show";//视频教育页在展示
  static const String ACTION_MANUAL_DEBUG = "action_mannual_debug";
  static const String ACTION_SHOWAD_TRACK = "action_showad_track";//广告展现回传
  static const String ACTION_STAYSCREEN_LIGHT = "action_stayscreen_light";//保持屏幕长亮或者恢复正常
  static const String ACTION_CACHE_VIDEO = "action_cache_video";//缓存videoplayer

  static const int STATISTICS_COUNTING_TYPE = 1;/// 计数类型的统计数据
  static const int STATISTICS_IMMEDIATELY_TYPE = 2;/// 即时发送类型的统计数据

  /**
  界面跳转
   */
  static Future<bool> navigator(String scheme, String param) async{
    bool result = false;
    try{
      result = await _platformAction.invokeMethod(ACTION_NAVIGATOR, {"scheme":scheme, "param":param});
    } on PlatformException catch (e) {
      ILog.lg.e("Invoked platform fun failed: navigator E:" + e.toString());
    }

    return result;
  }

  /**
   * 数据统计
   */
  static Future<bool> statistics(int type, dynamic param) async{
    // 默认添加 'time' 字段
    if(param is Map){
      Map content = param['param'];
      if(content is Map) {
        param['time'] = StatisticsHelper.getCurrentTime();
      }
    }
    bool result = false;
    try{
      result = await _platformAction.invokeMethod(ACTION_STATISTICS, {"type":type, "param":param});
    } on PlatformException catch (e) {
      ILog.lg.e("Invoked platform fun failed: statistics E:" + e.toString());
    }

    return result;
  }

  /**
   * crash收集
   */
  static Future<bool> whenCrash(String extra, String trace) async{
    Map<String, String> param = new HashMap();
    param["extra"] = extra;
    param["trace"] = trace;
    bool result = false;
    try{
      result = await _platformAction.invokeMethod(ACTION_WHEN_CRASH, param);
    } on PlatformException catch (e) {
      ILog.lg.e("Invoked platform fun failed: statistics E:" + e.toString());
    }

    return result;
  }

  /// 获取系统信息
  static Future<Map> getSysInfo() async {
    Map<dynamic, dynamic> mapRes;

    try {
      mapRes = await _platformAction.invokeMethod(ACTION_GETSYSINFO);
      assert(mapRes != null);
      return mapRes;
    } on PlatformException catch (e) {
      ILog.lg.e("Invoked platform fun failed: getSysInfo E:" + e.toString());
    }
    return null;
  }


  /**
   * 视频界面关闭
   */
  static Future<bool> pageClose(Map param) async{
    bool result = false;
    try{
      result = await _platformAction.invokeMethod(ACTION_PAGECLOSE, param);
    } on PlatformException catch (e) {
      ILog.lg.e("Invoked platform fun failed: statistics E:" + e.toString());
    }

    return result;
  }

  /**
   * 投诉反馈
   */
  static Future<int> feedback(Map param) async{
    int result = 0;
    try{
      result = await _platformAction.invokeMethod(ACTION_FEEDBACK, param);
    } on PlatformException catch (e) {
      ILog.lg.e("Invoked platform fun failed: statistics E:" + e.toString());
    }

    return result;
  }


  /**
   * 不感兴趣
   */
  static Future<String> userNotLike(Map param) async{
    String result = "";
    try{
      result = await _platformAction.invokeMethod(ACTION_NOTLIKE, param);
    } on PlatformException catch (e) {
      ILog.lg.e("Invoked platform fun failed: statistics E:" + e.toString());
    }

    return result;
  }

  /**
   * 收藏
   */
  static Future<dynamic> favorite(Map param) async{
    dynamic result;
    try{
      result = await _platformAction.invokeMethod(ACTION_FAVORITE, param);
    } on PlatformException catch (e) {
      ILog.lg.e("Invoked platform fun failed: statistics E:" + e.toString());
    }

    return result;
  }

  /**
   * 查看广告详情
   */
  static Future<bool> openAdDetail(Map param) async{
    bool result = false;
    try{
      result = await _platformAction.invokeMethod(ACTION_OPENADDETAIL, param);
    } on PlatformException catch (e) {
      ILog.lg.e("Invoked platform fun failed: statistics E:" + e.toString());
    }

    return result;
  }

  /**
   * 立即下载
   */
  static Future<bool> downloadAdApp(Map param) async{
    bool result = false;
    try{
      result = await _platformAction.invokeMethod(ACTION_DOWNLOADADAPP, param);
    } on PlatformException catch (e) {
      ILog.lg.e("Invoked platform fun failed: statistics E:" + e.toString());
    }

    return result;
  }
  /**
   * 广告展现回传
   */
  static Future<bool> showAdSendTrack(Map param) async{
    bool result = false;
    try{
      result = await _platformAction.invokeMethod(ACTION_SHOWAD_TRACK, param);
    } on PlatformException catch (e) {
      ILog.lg.e("Invoked platform fun failed: statistics E:" + e.toString());
    }

    return result;
  }

  /**
   * 视频点赞
   */
  static Future<int> giveLike(Map param) async{
    int result = 0;
    try{
      result = await _platformAction.invokeMethod(ACTION_GIVELIKE, param);
    } on PlatformException catch (e) {
      ILog.lg.e("Invoked platform fun failed: statistics E:" + e.toString());
    }

    return result;
  }

  /**
   * 展示评论
   * 回传评论条目！！
   */
  static Future<Map> showComments(Map param) async{
    Map result = Map();
    try{
      result = await _platformAction.invokeMethod(ACTION_SHOWCOMMENTS, param);
    } on PlatformException catch (e) {
      ILog.lg.e("Invoked platform fun failed: statistics E:" + e.toString());
    }

    return result;
  }

  /**
   * 分享
   */
  static Future<bool> shareVideo(Map param) async{
    bool result = false;
    try{
      result = await _platformAction.invokeMethod(ACTION_SHARE, param);
    } on PlatformException catch (e) {
      ILog.lg.e("Invoked platform fun failed: statistics E:" + e.toString());
    }

    return result;
  }

  /**
   * 写评论
   */
  static Future<Map> writeComments(Map param) async{
    Map result = Map();
    try{
      result = await _platformAction.invokeMethod(ACTION_WRITECOMMENTS, param);
    } on PlatformException catch (e) {
      ILog.lg.e("Invoked platform fun failed: statistics E:" + e.toString());
    }

    return result;
  }

  /**
   * 视频切换
   */
  static Future<bool> switchVideo(Map param) async{
    bool result = false;
    try{
      result = await _platformAction.invokeMethod(ACTION_SWITCH_VIDEO, param);
    } on PlatformException catch (e) {
      ILog.lg.e("Invoked platform fun failed: statistics E:" + e.toString());
    }

    return result;
  }

  /**
   * 视频教育页
   */
  static Future<int> educationPageShow(Map param) async{
    int result = 1;
    try{
      result = await _platformAction.invokeMethod(ACTION_EDUCATION_SHOW, param);
    } on PlatformException catch (e) {
      ILog.lg.e("Invoked platform fun failed: statistics E:" + e.toString());
    }

    return 1;
  }

  static Future<String> manualDebug(String orgMsg) async{
    ILog.lg.invoke("manualDebug Start");
    String result = "";
    try{
      result = await _platformAction.invokeMethod(ACTION_MANUAL_DEBUG, orgMsg);
    } on PlatformException catch (e) {
      ILog.lg.e("Invoked platform fun failed: statistics E:" + e.toString());
    }

    ILog.lg.invoke("manualDebug End");
    return result;
  }

  /**
   * 设置屏幕长亮或者自然状态
   */
  static Future<int> stayScreenLight(bool param) async{
    int result = 0;
    try{
      result = await _platformAction.invokeMethod(ACTION_STAYSCREEN_LIGHT, param);
    } on PlatformException catch (e) {
      ILog.lg.e("Invoked platform fun failed: statistics E:" + e.toString());
    }

    return result;
  }


  /**
   * 设置屏幕长亮或者自然状态
   */
  static Future<int> cacheVideo(List<String> urls, bool append) async{
    int result = 0;
    Map param = Map();
    param["urls"] = urls;
    param["append"] = append;
    try{
      result = await _platformAction.invokeMethod(ACTION_CACHE_VIDEO, param);
    } on PlatformException catch (e) {
      ILog.lg.e("Invoked platform fun failed: statistics E:" + e.toString());
    }

    return result;
  }
}

