import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:short_video/base/i_log.dart';
import 'package:short_video/model_item.dart';
import 'package:short_video/model_main.dart';
import 'package:short_video/platform/sg_operation.dart';
import 'package:short_video/util/constant.dart';
import 'package:short_video/util/ui_tools.dart';

import 'image_button_widget.dart';

class CommentCountWidget extends StatelessWidget {
  final VideoItemModel _itemModel;
  DateTime _lastClickTime;

  CommentCountWidget(this._itemModel);

  @override
  Widget build(BuildContext context) {
    return Column(
      children: <Widget>[
        ImageButtonWidget(
          normalImage: "assets/images/video_toolbar_comment_normal.png",
          highlightedImage:
              "assets/images/video_toolbar_comment_highlighted.png",
          width: 40,
          height: 40,
          status: ImageButtonStatus.normal,
          onTap: () {
            _clickCommentVideoButtonWidget(context);
          },
        ),
        SizedBox(height:3),
        Text(
          UITools.formatNum(_itemModel.currCommentCount),
          textAlign: TextAlign.center,
          style: TextStyle(fontSize: 14, color: Colors.white),
        ),
//        SizedBox(
//          height: 20.0,
//          child: Container(
//            margin: EdgeInsets.only(left: 0, top: 3),
//            color: Colors.transparent,
//            child: Text(
//              UITools.formatNum(_itemModel.currCommentCount),
//              textAlign: TextAlign.center,
//              style: TextStyle(fontSize: 14, color: Colors.white),
//            ),
//          ),
//        ),
      ],
    );
  }

//调起评论展示视图事件
  void _clickCommentVideoButtonWidget(BuildContext context) {
//    print("评论的文章ID"+topicId);
    if (_lastClickTime != null &&
        DateTime.now().difference(_lastClickTime) <= Duration(seconds: 1)) {
      ILog.lg.i("评论时间:$_lastClickTime");
      return;
    } else {
      _lastClickTime = DateTime.now();
    }
    int _playOrderRecord = MainModel.of(context).getPlayOrder();
    MainModel.of(context).setPlayOrder(MainModel.ORDER_TYPE_LOOP_FOR_COMMENTS);
    ILog.lg.i("当前状态:$_playOrderRecord");
    Map param = new Map();
    param['newsid'] = _itemModel.videoItem.newsid ?? "";
    param['url'] = _itemModel.videoItem.url ?? "";
    param['title'] = _itemModel.videoItem.title ?? "";
    param['key'] = _itemModel.videoItem.key ?? "";
    param['from'] = _itemModel.videoItem.from ?? "";
    param['topic'] = _itemModel.videoItem.topic ?? "";
    param['input'] = _itemModel.commentText ?? "";
    param['commentCount'] = _itemModel.currCommentCount ?? 0;
    SgOperation.showComments(param).then((value) {
      MainModel.of(context).setPlayOrder(_playOrderRecord);
      ILog.lg.i("结束状态:$_playOrderRecord");
      if (value is Map) {
        ILog.lg.i("评论回传数据:$value");
        String input = value["input"];
        int staytime = value["staytime"];
        int commentCount = value["commentCount"];
        assert(staytime != null);
        // 保存本次评论内容，等待评论切换时，清空；
        _itemModel.onNativeCommentResult(input, commentCount);
        String pingbackKey = "InformStayTime";
        Map pingbackParam = new Map();
        pingbackParam["StayModule"] = "shortvideolist";
        pingbackParam["Channel"] = Constant.CHANNEL ?? "小视频";
        pingbackParam["page-type"] = "video";
        String urlTemp = _itemModel.videoItem.url ?? "";
        Uri u = Uri.parse(urlTemp);
        String newsType = "other";
        if (u.host != null && u.host.toLowerCase().startsWith("bazinga")) {
          newsType = "bazinga";
        }
        pingbackParam["news-type"] = newsType;
        pingbackParam["newsid"] = _itemModel.videoItem.newsid;
        pingbackParam["CommentStayTime"] = staytime;
        Map dataMap = new Map();
        dataMap["key"] = pingbackKey;
        dataMap["param"] = pingbackParam;
        SgOperation.statistics(
            SgOperation.STATISTICS_IMMEDIATELY_TYPE, dataMap);
      }
    });
    //pingback
    String pingbackKey = "ShortVideoCommentClick";
    Map pingbackParam = new Map();
    pingbackParam["name"] = Constant.CHANNEL ?? "";
    pingbackParam["url"] = _itemModel.videoItem.url ?? "";
    pingbackParam["title"] = _itemModel.videoItem.title ?? "";
    pingbackParam["newsid"] = _itemModel.videoItem.newsid ?? "";
    pingbackParam["activetype"] = "button";
    Map dataMap = new Map();
    dataMap["key"] = pingbackKey;
    dataMap["param"] = pingbackParam;
    SgOperation.statistics(SgOperation.STATISTICS_IMMEDIATELY_TYPE, dataMap);
  }
}
