import 'package:flutter/material.dart';
import 'package:short_video/base/i_log.dart';

/// 淡入淡出，切换widget
class FadeInWidgetSwitcher extends StatefulWidget {
  final Widget child;
  final Duration duration;

  const FadeInWidgetSwitcher(
      {Key key, this.duration = const Duration(milliseconds: 300), this.child})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _FadeInWidgetSwitcherState();
  }
}

class _FadeInWidgetSwitcherState extends State<FadeInWidgetSwitcher>
    with SingleTickerProviderStateMixin {
  AnimationController animationController;

  @override
  void initState() {
    super.initState();
    animationController =
        AnimationController(duration: widget.duration, vsync: this);
    animationController.addListener(() {
      if (mounted) {
        setState(() {});
      }
    });
    animationController.forward(from: 0.0);
  }

  @override
  void deactivate() {
    animationController.stop();
    super.deactivate();
  }

  @override
  void didUpdateWidget(FadeInWidgetSwitcher oldWidget) {
    super.didUpdateWidget(oldWidget);
    ILog.lg.i("oldWidget = $oldWidget");
    if (oldWidget.child != widget.child) {
      animationController.forward(from: 0.0);
    }
  }

  @override
  void dispose() {
    animationController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return animationController.isAnimating
        ? Transform.scale(
            scale: 2 - animationController.value,
            child: Opacity(
              opacity: animationController.value,
              child: widget.child,
            ),
          )
        : widget.child;
  }
}
