import 'package:dio/dio.dart';
import 'package:flutter/material.dart';
import 'package:short_video/debug/debug_screen.dart';

import '../build_config.dart';
import '../model_item.dart';

/// loading、error页面

class VideoPageError extends StatelessWidget {
  static const int STATE_LOADING = 1;
  static const int STATE_ERROR = 2;
  static const int STATE_NONET = 3;

  final int state;
  final VoidCallback clickCallback;

  VideoPageError(this.state, this.clickCallback);

  @override
  Widget build(BuildContext context) {
    if (state == STATE_LOADING) {
      return Center(
        child: CircularProgressIndicator(),
      );
    } else {
      return Center(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            _buildDebugButton(context),
            IconButton(
              icon: Icon(Icons.refresh),
              iconSize: 80.0,
              color: Colors.white,
              onPressed: clickCallback,
            ),
            Text(
              getTipByState(state),
              style: TextStyle(color: Colors.white, fontSize: 15),
            ),
          ],
        ),
      );
    }
  }
  Widget _buildDebugButton(BuildContext context) {
//    int itemIndex = VideoItemModel.of(context).index;

    return BuildConfig().needDebugMenu()
        ? RaisedButton(
      child: Text("Debug"),
      onPressed: () {
        Navigator.push(
          context,
          MaterialPageRoute(builder: (context) => DebugScreen()),
        );
      },
    )
        : Container();
  }
  String getTipByState(int state) {
    if (state == STATE_ERROR) {
      return "加载失败，请稍后再试";
    } else if (state == STATE_NONET) {
      return "网络异常，请稍后再试";
    }
    return "";
  }
}
