import 'package:flutter/material.dart';
import 'package:short_video/base/i_log.dart';
import 'dart:math';
import 'package:short_video/bean/heart_info.dart';
import 'package:flutter/painting.dart';
import 'dart:ui';

//import 'dart:typed_data';
//import 'dart:async';
import 'dart:ui' as ui
    show instantiateImageCodec, Codec, Image, TextStyle, PictureRecorder;
import 'package:flutter/services.dart';

/// 绘制心形动画
class GiveLikeAnimationScreen extends StatefulWidget {
  static ui.Image _heartImage;
  static ui.Image _heartLeftImage;
  static ui.Image _heartRightImage;
  static bool _imageLoading = false;

  final ListHeartValeNotifier observer;

  GiveLikeAnimationScreen(this.observer) : assert(observer != null);

  @override
  State<StatefulWidget> createState() {
    return _GiveLikeAnimationScreenState();
  }

  static Future<void> loadHeartImage() async {
    _imageLoading = true;
    _heartImage =
        await loadLocalImage("./assets/images/video_heartbeat_normal.png");
    _heartLeftImage =
        await loadLocalImage("./assets/images/video_heartbeat_left.png");
    _heartRightImage =
        await loadLocalImage("./assets/images/video_heartbeat_right.png");
    assert(_heartImage != null);
    assert(_heartLeftImage != null);
    assert(_heartRightImage != null);
    _imageLoading = false;
  }

  // 通过assets路径，获取资源图片
  static Future<ui.Image> loadLocalImage(String assets) async {
    final ByteData data = await rootBundle.load(assets);
    if (data == null) throw 'Unable to read data';
    Codec codec = await ui.instantiateImageCodec(data.buffer.asUint8List());
    FrameInfo frame = await codec.getNextFrame();
    return frame.image;
  }
}

class _GiveLikeAnimationScreenState extends State<GiveLikeAnimationScreen>
    with SingleTickerProviderStateMixin {
  AnimationController _controller;
  Animation<Color> animation;

  @override
  void initState() {
    if (GiveLikeAnimationScreen._heartImage == null &&
        !GiveLikeAnimationScreen._imageLoading) {
      GiveLikeAnimationScreen.loadHeartImage().then((_) {
        if (mounted) {
          setState(() {});
        }
      });
    }
    if (widget.observer != null) {
      widget.observer.addListener(_handleValueChanged);
    }
    _controller = new AnimationController(
      duration: const Duration(milliseconds: 1150),
      vsync: this,
    );
    _controller.addListener(() {
      if (mounted) {
        setState(() {});
      }
    });
    _controller.addStatusListener((AnimationStatus status) {
//      print("动画状态"+status.toString());
      if (status == AnimationStatus.completed) {
        if (mounted) {
          setState(() {});
        }
      } else if (status == AnimationStatus.forward) {}
    });
    super.initState();
  }

  @override
  void dispose() {
    if (widget.observer != null) {
      widget.observer.removeListener(_handleValueChanged);
    }
    _controller.dispose();
    super.dispose();
  }

  // 通过assets路径，获取资源图片
  Future<ui.Image> loadLocalImage(String assets) async {
    final ByteData data = await rootBundle.load(assets);
    if (data == null) throw 'Unable to read data';
    Codec codec = await ui.instantiateImageCodec(data.buffer.asUint8List());
    FrameInfo frame = await codec.getNextFrame();
    return frame.image;
  }

  @override
  Widget build(BuildContext context) {
    if (GiveLikeAnimationScreen._heartImage == null) {
      return Container();
    } else {
      return CustomPaint(
          size: Size.infinite,
          painter: VideoHeartBeatPainter(
              widget.observer.value,
              GiveLikeAnimationScreen._heartImage,
              GiveLikeAnimationScreen._heartLeftImage,
              GiveLikeAnimationScreen._heartRightImage));
    }
  }

  void _handleValueChanged() {
    if (mounted) {
      setState(() {});
    }
    if (widget.observer != null && widget.observer.value.length > 0) {
      _controller.reset();
      _controller.forward();
    }
  }
}

/**
 * 点赞动画：
 * 第一步. 放大0%-108% 0.08秒
 * 第二步. 缩小108%-100%，0.07秒
 * 第三步. 停顿0.5秒
 * 第四步. 放大100%-150%透明度100%-0%0.5秒
 */
class VideoHeartBeatPainter extends CustomPainter {
  Paint pt = Paint();
  List<HeartInfo> heartList;

  ui.Image heartImage;

  ui.Image heartLeftImage;

  ui.Image heartRightImage;

  VideoHeartBeatPainter(this.heartList, this.heartImage, this.heartLeftImage,
      this.heartRightImage);

  @override
  void paint(Canvas canvas, Size size) {
//    ILog.lg.i("screen size= .$size ");
    double animDur = 1150; //总时长
    int currTime = DateTime.now().millisecondsSinceEpoch;
    pt.color = Colors.red;
    List<HeartInfo> removed = List<HeartInfo>();
    heartList.forEach((HeartInfo hi) {
//      print("paint currTime - hi.cTime: ${currTime - hi.cTime} ");
      if (currTime - hi.cTime >= animDur) {
        removed.add(hi);
      } else {
        double animRate = (currTime - hi.cTime) / animDur;
        double opacity = getImageOpactityWithTimePercent(
            animRate, (currTime - hi.cTime) * 1.0, animDur);
        pt.color = Color.fromRGBO(255, 0, 0, opacity);
        double scale = getImageScaleWithTimePercent(
            animRate, (currTime - hi.cTime) * 1.0, animDur);
        ui.Image targetImage;
        if (hi.angle < 0) {
          targetImage = heartLeftImage;
        } else if (hi.angle > 0) {
          targetImage = heartRightImage;
        } else {
          targetImage = heartImage;
        }
        canvas.drawImageRect(
            targetImage,
            Rect.fromLTWH(0, 0, 282, 252),
            Rect.fromLTWH(hi.posX - 50 * scale, hi.posY - 50 * scale,
                282/3 * scale, 252/3 * scale),
            pt);
      }
    });
    removed.forEach((needRemoved) {
      heartList.remove(needRemoved);
    });
  }

  // 根据时间获取图像的展示比例如0.6倍、1倍、2倍
  // progress 当前时间进度百分比，curTime 动画执行的时间，totalTime动画总时长
  double getImageScaleWithTimePercent(
      double progress, double curTime, double totalTime) {

    //第一步. 放大0%-108% 0.08秒
    if (progress <= 0.08/1.15) {
      return 1.08*(curTime/80);

      //第二步. 缩小108%-100%，0.07秒
    } else if (progress > 0.08/1.15 && progress <= 0.15/1.15) {
      return 1.08-(curTime-80)/70*0.08;

      //第三步. 停顿0.5秒
    } else if (progress > 0.15/1.15 && progress <= 0.65/1.15) {
      return 1.0;

      //第四步. 放大100%-150%透明度100%-0%0.5秒
    } else if (progress > 0.65/1.15 && progress <= 1){
      return 1.0 + (curTime-650)/500*0.7;
    } else {
      return 2;
    }
  }

  // 根据时间获取当前图像的不透明度
  // progress 当前时间进度百分比，curTime 动画执行的时间，totalTime动画总时长
  double getImageOpactityWithTimePercent(
      double progress, double curTime, double totalTime) {
    //第一步. 放大0%-108% 0.08秒
    if (progress <= 0.65/1.15) {
      return 1.0;

      //第四步. 放大100%-150%透明度100%-0%0.5秒
    } else if (progress > 0.65/1.15 && progress <= 1) {
      return 1.0 -(curTime-650)/500;
    } else {
      return 0.0;
    }
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
//    print("shouldRepaint ... ");
    return true;
  }
}

class ListHeartValeNotifier extends ValueNotifier<List<HeartInfo>> {
  ListHeartValeNotifier(List<HeartInfo> value) : super(value);

  //变更双击点的数目
  void changeVideoHeartInfoNumber(HeartInfo heart) {
    value.add(heart);
    notifyListeners();
  }
}
