import 'dart:convert';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:short_video/base/i_log.dart';
import 'package:short_video/bean/video_item.dart';
import 'package:short_video/build_config.dart';
import 'package:short_video/debug/debug_screen.dart';
import 'package:short_video/main_screen.dart';
import 'package:short_video/model_item.dart';
import 'package:short_video/model_main.dart';
import 'package:short_video/platform/sg_operation.dart';
import 'package:short_video/util/constant.dart';
import 'package:short_video/widget/ad_text_widget.dart';
import 'package:short_video/widget/popup_menu_widget.dart';

import 'image_button_widget.dart';

class NaviBarWidget extends StatelessWidget {
  final VideoItem videoItem;
  final String channelId; //分类channelid 如小视频，推荐
  NaviBarWidget({Key key, this.videoItem, this.channelId}) : super(key: key);

  int _playOrderRecord; //记录视频播放的模式

  @override
  Widget build(BuildContext context) {
    return Align(
      //返回按钮和更多
      alignment: Alignment.topCenter,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          GestureDetector(
            child: Container(
              color: Colors.transparent,
              child: Container(
                margin: EdgeInsets.only(left: 10, top: Constant.getNaviTop()),
                width: 40,
                height: Constant.NAVI_HEIGHT,
                alignment: Alignment.centerLeft,
                child: ImageButtonWidget(
                  normalImage: "assets/images/video_navi_back_normal.png",
                  highlightedImage: "assets/images/video_navi_back_press.png",
                  width: 32,
                  height: 32,
                  imageWidth: 32,
                  imageheight: 22,
                  status: ImageButtonStatus.normal,
                  onTap: () {
                    clickNaivBackButtonWidget(context);
                  },
                ),
              ),
            ),
            onTapDown: (d) {},
          ),
          _buildDebugButton(context),
          Container(
            width: 43,
            height: Constant.NAVI_HEIGHT,
            color: Colors.transparent,
            margin: EdgeInsets.only(right: 10, top: Constant.getNaviTop()),
            child: ImageButtonWidget(
              normalImage: "assets/images/video_navi_more_normal.png",
              highlightedImage: "assets/images/video_navi_more_press.png",
              width: 32,
              height: 32,
              imageWidth: 32,
              imageheight: 32,
              status: ImageButtonStatus.normal,
              onTap: () {
                clickNaviMoreButtonWidget(context);
              },
            ),
          ),
        ],
      ),
    );
  }

//右上角 菜单内容widget
  Widget createTopRightMoreMenuWidget(bool showFavorite, BuildContext context) {
    double widthTemp = 125;
    double heightTemp = 128;
    if (!showFavorite) {
      heightTemp = 88;
    }
    return new Container(
      width: widthTemp,
      height: heightTemp,
      color: Colors.transparent,
      child: CustomPaint(
        child: Container(
          margin: EdgeInsets.only(top: 8),
          child: Column(
            children: <Widget>[
              GestureDetector(
                child: Container(
                  width: widthTemp,
                  height: 40,
                  color: Colors.transparent,
                  child: Row(
                    crossAxisAlignment: CrossAxisAlignment.center,
                    children: <Widget>[
                      Container(
                        width: 30,
                        height: 30,
                        margin: EdgeInsets.all(5),
                        child: Image.asset(
                          "assets/images/video_menu_feedback_normal.png",
                          fit: BoxFit.fill,
                        ),
                      ),
                      Container(
                        width: widthTemp - 45,
                        height: 30,
                        alignment: Alignment.centerLeft,
                        padding: EdgeInsets.only(left: 10),
                        child: Text(
                          '投诉',
                          textAlign: TextAlign.left,
                          style: TextStyle(
                            fontSize: 15,
                            color: Colors.black,
                          ),
                        ),
                      ),
                    ],
                  ),
                ),
                onTap: () {
                  ILog.lg.i("投诉");
                  Navigator.of(context).pop();
                  clickMenuFeedBackWidget(context);
                },
              ),
              GestureDetector(
                child: Container(
                  width: widthTemp,
                  height: 40,
                  color: Colors.transparent,
                  child: Row(
                    crossAxisAlignment: CrossAxisAlignment.center,
                    children: <Widget>[
                      Container(
                        width: 30,
                        height: 30,
                        margin: EdgeInsets.all(5),
                        child: Image.asset(
                          "assets/images/video_menu_dislike_normal.png",
                          fit: BoxFit.fill,
                        ),
                      ),
                      Container(
                        width: widthTemp - 45,
                        height: 30,
                        padding: EdgeInsets.only(left: 10),
                        alignment: Alignment.centerLeft,
                        child: Text(
                          '不感兴趣',
                          textAlign: TextAlign.left,
                          style: TextStyle(
                            fontSize: 15,
                            color: Colors.black,
                          ),
                        ),
                      ),
                    ],
                  ),
                ),
                onTap: () {
                  ILog.lg.i("不感兴趣");
                  Navigator.of(context).pop();
                  clickMenuDislikeWidget(context);
                },
              ),
              showFavorite
                  ? GestureDetector(
                      child: Container(
                        width: widthTemp,
                        height: 40,
                        color: Colors.transparent,
                        child: Row(
                          crossAxisAlignment: CrossAxisAlignment.center,
                          children: <Widget>[
                            Container(
                              width: 30,
                              height: 30,
                              margin: EdgeInsets.all(5),
                              child: Image.asset(
                                "assets/images/video_menu_favorite_normal.png",
                                fit: BoxFit.fill,
                              ),
                            ),
                            Container(
                              width: widthTemp - 45,
                              height: 30,
                              padding: EdgeInsets.only(left: 10),
                              alignment: Alignment.centerLeft,
                              child: Text(
                                '收藏',
                                textAlign: TextAlign.left,
                                style: TextStyle(
                                  fontSize: 15,
                                  color: Colors.black,
                                ),
                              ),
                            ),
                          ],
                        ),
                      ),
                      onTap: () {
                        Navigator.of(context).pop();
                        clickMenuIosFavoriteWidget(context);
                      },
                    )
                  : Container(),
            ],
          ),
        ),
        painter: PopupMenuBorderPainter(),
      ),
    );
  }

  Widget _buildDebugButton(BuildContext context) {
    int itemIndex = VideoItemModel.of(context).index;

    return BuildConfig().needDebugMenu()
        ? RaisedButton(
            child: Text("Debug-$itemIndex"),
            onPressed: () {
              Navigator.push(
                context,
                MaterialPageRoute(builder: (context) => DebugScreen()),
              );
            },
          )
        : Container();
  }

  //左上角返回
  clickNaivBackButtonWidget(BuildContext context) async {
    Map param = new Map();
    param["newsid"] = videoItem.newsid ?? "";
    SgOperation.pageClose(param).then((value){
      MainModel.of(context).dispose();
    });
  }

//右上角更多菜单
  void clickNaviMoreButtonWidget(BuildContext context) {
    _playOrderRecord = MainModel.of(context).getPlayOrder();
    ILog.lg.i("导航更多功能:$_playOrderRecord");
    MainModel.of(context).setPlayOrder(MainModel.ORDER_TYPE_LOOP_FOR_COMMENTS);
    ILog.lg.i("当前状态:$_playOrderRecord");
    bool showFavorite = false;
    if (Platform.isIOS) {
      showFavorite = true;
    }
    if (videoItem.category != null &&
        (videoItem.adtype == 1 || videoItem.adtype == 0)) {
      showFavorite = false; //广告不展示收藏
    }
    BuildContext mainCtx = MainScreen.mainScreenKey.currentContext;
    Navigator.push(
      mainCtx,
      VideoPopRoute(
        child: VideoPopup(
          child: createTopRightMoreMenuWidget(showFavorite, context),
          left: Adapt.screenW() - 10 - 125,
          // -右边距-控件宽度
          top: Constant.getNaviTop() + Constant.NAVI_HEIGHT,
          onClick: () {
            ILog.lg.i("exit退出");
          },
          backgroundClick: () {
            ILog.lg.i("点击遮罩背景：$_playOrderRecord");
            MainModel.of(context).setPlayOrder(_playOrderRecord);
          },
        ),
      ),
    );
  }

  //  更多菜单-投诉
  void clickMenuFeedBackWidget(BuildContext context) {
    print("投诉");
    moreMenuFeedbackClick(context, _playOrderRecord);
    Map param = new Map();
    param["url"] = videoItem.url??"";
    param["key"] = videoItem.key??"";
    param["title"] = videoItem.title??"";
    param["newsid"] = videoItem.newsid??"";
    SgOperation.feedback(param);
    //pingback
    ILog.lg.i("点击投诉" + videoItem.toString());
    String pingbackKey = "ShortVideoMenuItemClick";
    Map pingbackParam = new Map();
    pingbackParam["name"] = channelId ?? "";
    pingbackParam["url"] = videoItem.url ?? "";
    pingbackParam["title"] = videoItem.title ?? "";
    pingbackParam["newsid"] = videoItem.newsid ?? "";
    pingbackParam["itemname"] = "feedback";
    Map dataMap = new Map();
    dataMap["key"] = pingbackKey;
    dataMap["param"] = pingbackParam;
    SgOperation.statistics(SgOperation.STATISTICS_IMMEDIATELY_TYPE, dataMap);
  }

//  更多菜单-不感兴趣
  void clickMenuDislikeWidget(BuildContext context) {
    List<dynamic> myList = videoItem.reason;
    //广告不感兴趣的内容为空，仅仅展示不感兴趣按钮
    if (myList == null || !(myList is List)) {
      myList =  List<dynamic>();
    }
    
    Map param = new Map();
    param['newsid'] = videoItem.newsid ?? "";
    param["channelid"] = "小视频";
    param["topic"] = videoItem.topic ?? "";
    param["publishtime"] = videoItem.timestampms ?? "";
    param["key"] = videoItem.key ?? "";
    param['reason'] = myList;
    SgOperation.userNotLike(param).then((value) {
      if (value is String) {
        ILog.lg.i("不感兴趣数据的newsid=" + value);
        //点击播放列表页的不感兴趣，不关闭当前的视频，这里不执行删除了
        // 执行本地删除
//        MainModel.of(context).removeVideoItem(videoItem);
      }
      MainModel.of(context).setPlayOrder(_playOrderRecord); //恢复播放顺序
    });
    //pingback
    String pingbackKey = "ShortVideoMenuItemClick";
    Map pingbackParam = new Map();
    pingbackParam["name"] = channelId ?? "";
    pingbackParam["url"] = videoItem.url ?? "";
    pingbackParam["title"] = videoItem.title ?? "";
    pingbackParam["newsid"] = videoItem.newsid ?? "";
    pingbackParam["itemname"] = "nointerest";
    Map dataMap = new Map();
    dataMap["key"] = pingbackKey;
    dataMap["param"] = pingbackParam;
    SgOperation.statistics(SgOperation.STATISTICS_IMMEDIATELY_TYPE, dataMap);
  }

//  更多菜单-收藏
  void clickMenuIosFavoriteWidget(BuildContext context) {
    MainModel.of(context).setPlayOrder(_playOrderRecord); //恢复播放顺序
    if (videoItem.url == null || videoItem.url.length < 1) {
      return; //不存在则不收藏
    }
    ILog.lg.i("iOS收藏");
    Map param = new Map();
    //类型
    param["type"] = videoItem.showtype ?? "";
    //标题
    param['title'] = videoItem.title ?? "";
    //发布时间
    param['publishTime'] = (videoItem.timestampms ?? "").toString();
    //来源
    param['source'] = videoItem.from ?? "";
    //资讯id
    param['gid'] = videoItem.newsid ?? "";

    if (videoItem.picurls != null && videoItem.picurls.length > 0) {
      param['thumbUrl'] = videoItem.picurls[0];
      param['imageNum'] = videoItem.picurls.length;
    } else {
      //缩略图
      param['thumbUrl'] = "";
      //图片数量
      param['imageNum'] = 0;
    }
    //视频时长
    param['videoTime'] = videoItem.video_time ?? "";
    //url
    param['url'] = videoItem.url ?? "";
    SgOperation.favorite(param).then((res) {
      if (res != null && res is String) {
        print("客户端收藏回传结果=" + res);
        Map resMap = jsonDecode(res);
        if (resMap != null) {
          String method = resMap["method"];
          bool status = resMap["staus"];
          if (method == "addfavorite") {
            if (status == true) {
              //收藏成功
            } else {
              //添加收藏失败，提示在客户端进行
            }
          } else if (method == "cancelfavorite") {
            if (status == true) {
              //取消收藏成功
            } else {
              //取消收藏失败，提示在客户端进行
            }
          }
        }
      }
    });
    //pingback
    String pingbackKey = "ShortVideoMenuItemClick";
    Map pingbackParam = new Map();
    pingbackParam["name"] = channelId ?? "";
    pingbackParam["url"] = videoItem.url ?? "";
    pingbackParam["title"] = videoItem.title ?? "";
    pingbackParam["newsid"] = videoItem.newsid ?? "";
    pingbackParam["itemname"] = "collect";
    Map dataMap = new Map();
    dataMap["key"] = pingbackKey;
    dataMap["param"] = pingbackParam;
    SgOperation.statistics(SgOperation.STATISTICS_IMMEDIATELY_TYPE, dataMap);
  }

  void moreMenuFeedbackClick(BuildContext context, int playorder) {
    // 无需手动暂停，didChangeAppLifecycleState 会自动出发，暂停视频
    ILog.lg
        .i("反馈" + context.toString() + ";播放顺序：" + _playOrderRecord.toString());
    MainModel.of(context).setPlayOrder(_playOrderRecord); //恢复播放顺序
    ILog.lg.i("反馈:视频暂停");
    Map param = new Map();
    param["url"] = videoItem.url ?? "";
    param["key"] = videoItem.key ?? "";
    param["title"] = videoItem.title ?? "";
    param["newsid"] = videoItem.newsid ?? "";
  }
}
