import 'dart:io';

import 'package:flutter/material.dart';
import 'package:like_button/like_button.dart';
import 'package:scoped_model/scoped_model.dart';
import 'package:short_video/base/i_log.dart';
import 'package:short_video/bean/support_bean.dart';
import 'package:short_video/model_item.dart';
import 'package:short_video/platform/sg_operation.dart';
import 'package:short_video/util/constant.dart';
import 'package:short_video/util/ui_tools.dart';
import 'package:short_video/widget/ad_text_widget.dart';
import 'package:short_video/widget/comment_count_widget.dart';
import 'package:short_video/widget/pure_text_widget.dart';
import 'package:short_video/widget/share_button_widget.dart';
import 'package:short_video/widget/write_comment_widget.dart';

import 'navi_bar_widget.dart';

enum AdClickAreaType { from, title, detail, download }

//该类是视频遮罩的UI展示
class VideoItemMaskWidget extends StatelessWidget {
  final VideoItemModel _videoItem;
  int commentStayTime = 0; //pingback统计评论展示时长

  VideoItemMaskWidget(this._videoItem); //记录上一次点击的时间，防止用户快速连续点击调用native

  @override
  // TODO: implement widget
  Widget build(BuildContext context) {
    return Column(
      //                  crossAxisAlignment: CrossAxisAlignment.baseline,
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: <Widget>[
        NaviBarWidget(
          videoItem: _videoItem.videoItem,
          channelId: Constant.CHANNEL,
        ),
        getVideoBottomWidget(
            context,
            _videoItem.videoItem.category,
            _videoItem.videoItem.adtype,
            _videoItem.videoItem.video_url,
            _videoItem.videoItem.from_icon,
            _videoItem.videoItem.from,
            _videoItem.videoItem.title),
      ],
    );
  }

  //小视频 没有category 参数；    非下载类广告  有category参数（默认值为-2） 且 adtype=0 ；    下载类广告  有category参数（默认值为-2） 且 adtype=1

  Align getVideoBottomWidget(BuildContext context, int category, int adtype,
      String url, String headUrl, String name, String desc) {
//    print("头像="+headUrl);
    //需要视频的ID，用户头像URL，用户名称，视频描述，评论数目，点赞数目，广告详情URL和广告立即下载URL
    if (category != null && adtype == 0) {
      return new Align(
        alignment: Alignment.bottomLeft,
        child: viedoAdSingleDetailWiget(
            context,
            _videoItem.videoItem.from_icon,
            _videoItem.videoItem.from,
            _videoItem.videoItem.title,
            _videoItem.videoItem.url),
      );
    } else if (category != null && adtype == 1) {
      return new Align(
        alignment: Alignment.bottomLeft,
        child: viedoAdDetailAndDownloadWiget(
            context,
            _videoItem.videoItem.from_icon,
            _videoItem.videoItem.from,
            _videoItem.videoItem.title,
            _videoItem.videoItem.url,
            _videoItem.videoItem.url), //缺少广告的app下载URL
      );
    } else {
      return Align(
        alignment: Alignment.bottomRight,
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          mainAxisSize: MainAxisSize.min,
          children: [
            Padding(
              padding: EdgeInsets.only(bottom: 20 + 14.0, left: 15),
              child: normalVideoBottomWidget(_videoItem.videoItem.from_icon,
                  _videoItem.videoItem.from, _videoItem.videoItem.title),
            ),
            WriteCommentWidget(Constant.CHANNEL)
          ],
        ),
//        ),
      );
    }
  }

  //正常视频用户区域(包括视频基本信息和右部分享框)
  Widget normalVideoBottomWidget(String url, String name, String desc) => Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        crossAxisAlignment: CrossAxisAlignment.end,
        children: [
          Container(
            child: Expanded(
              child: normalVideoInfoSection(url, name, desc),
            ),
          ),
          Container(
            width: 70,
            height: 229,
            child: ScopedModelDescendant<VideoItemModel>(
                builder: (context, child, model) {
              return Column(
                crossAxisAlignment: CrossAxisAlignment.center,
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  // 点赞,避免dispose之后，页面还会重构：onPause -> notify -> dispose -> 页面更新出错
                  model.notiSupportInfo == null
                      ? Container()
                      : ValueListenableBuilder<SupportBean>(
                          builder: (BuildContext context, SupportBean value,
                              Widget child) {
                            // 此处用model的数值，而不是用 value的数值；因为页面有可能被重建时，value内容没有及时更新；
                            return LikeButton(
                              size: 40,
                              countPostion: CountPostion.bottom,
                              likeCount: model.currSupportCount,
                              isLiked: model.isSupported,
                              likeCountAnimationType:
                                  LikeCountAnimationType.none,
                              countBuilder:
                                  (int likeCount, bool isLiked, String text) {
                                if (likeCount == null || likeCount < 1) {
                                  return Container();
                                } else {
                                  return Text(
                                    UITools.formatNum(likeCount),
                                    textAlign: TextAlign.center,
                                    style: TextStyle(
                                        fontSize: 14, color: Colors.white),
                                  );
                                }
                              },
                              likeBuilder: (bool isLiked) {
                                String imgPath;
                                if (isLiked) {
                                  imgPath =
                                      "assets/images/video_toolbar_like_select.png";
                                } else {
                                  imgPath =
                                      "assets/images/video_toolbar_like_normal.png";
                                }
                                return Image.asset(imgPath);
                              },
                              onTap: model.clickLikeVideoButtonWidget,
                            );
                          },
                          valueListenable: model.notiSupportInfo,
                        ),
                  // 评论
                  CommentCountWidget(model),
                  // 分享
                  ShareButton(model.videoItem),
                ],
              );
            }),
          ),
        ],
      );

  //更加文本的宽度,获取对应的字符串 超出显示...
  String getAdjustTitleText(String str, double textWidth) {
    if (str == null || !(str is String)) {
      return "";
    }
    TextSpan caculateSpan = new TextSpan(
      style: TextStyle(
        color: Colors.white,
        fontSize: 18,
        letterSpacing: 1,
        wordSpacing: 1,
        backgroundColor: Colors.transparent,
      ),
      text: str,
    );
    TextPainter caculateTp = new TextPainter(
      text: caculateSpan,
      textAlign: TextAlign.center,
      textDirection: TextDirection.ltr,
    );
    caculateTp.layout();
    double widthTemp = caculateTp.width;
    if (widthTemp <= textWidth) {
      return str;
    } else {
      TextSpan ellipsisSpan = new TextSpan(
        style: TextStyle(
          color: Colors.white,
          fontSize: 18,
          letterSpacing: 1,
          wordSpacing: 1,
          backgroundColor: Colors.transparent,
        ),
        text: '...',
      );
      TextPainter ellipsisTp = new TextPainter(
        text: ellipsisSpan,
        textAlign: TextAlign.center,
        textDirection: TextDirection.ltr,
      );
      ellipsisTp.layout();
      double ellipsisWidth = ellipsisTp.width; //省略号宽度
      String targetStr = "";
      double curWidth = ellipsisWidth;
      bool isFinish = false;
      str.runes.forEach((rune) {
        String charStr = new String.fromCharCode(rune);
        TextSpan charSpan = new TextSpan(
          style: TextStyle(
            color: Colors.white,
            fontSize: 18,
            letterSpacing: 0,
            wordSpacing: 0,
            backgroundColor: Colors.transparent,
          ),
          text: charStr,
        );
        TextPainter charTp = new TextPainter(
          text: charSpan,
          textAlign: TextAlign.center,
          textDirection: TextDirection.ltr,
        );
        charTp.layout();
        double charWidth = charTp.width;
        if (curWidth + charWidth >= textWidth) {
          isFinish = true;
        } else {
          targetStr = targetStr + charStr;
          curWidth = curWidth + charWidth;
        }
      });
      targetStr = targetStr + "...";
      return targetStr;
    }
  }

//  视频基本信息区 用户头像和名字
  Column normalVideoInfoSection(String url, String name, String desc) {
    bool isUrlValid = UITools.checkUrlStringIsValid(url);
    return Column(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.start,
//      textDirection: TextDirection.ltr,
      children: [
        Container(
          //用户信息区
//          color: Colors.transparent,
          alignment: Alignment.bottomLeft,
          margin: EdgeInsets.only(left: 0, right: 0, bottom: 15),
          child: Flex(
            direction: Axis.horizontal,
            children: <Widget>[
              //用户头像
              Container(
                width: isUrlValid ? 25 : 0,
                height: 25,
//                color: Colors.amberAccent,
                margin: EdgeInsets.only(left: 0),
//                child:  Container(),
                child: (url != null && url is String && url.length > 0)
                    ? Container(
                        margin: EdgeInsets.all(1),
                        decoration: BoxDecoration(
                          color: Colors.white,
                          shape: BoxShape.rectangle,
                          borderRadius: BorderRadius.circular(11.5),
                          image: DecorationImage(
                            fit: BoxFit.cover,
                            image: FadeInImage.assetNetwork(
                              placeholder:
                                  ('assets/images/video_user_default_icon.png'),
                              image: url,
                            ).image,
                          ),
                        ),
                      )
                    : Container(),
                decoration: BoxDecoration(
                  color: url != null ? Colors.white : Colors.transparent,
                  shape: BoxShape.rectangle,
                  borderRadius: BorderRadius.circular(12.5),
                ),
              ),
              Expanded(
                child: Container(
//                padding: const EdgeInsets.fromLTRB(7, 0, 5, 0),
//                color: Colors.transparent,
                  margin: EdgeInsets.only(
                      left: isUrlValid ? 7 : 0, right: 5, top: 0, bottom: 0),
                  height: 25,
                  child: Text(
                    //用户昵称
                    getAdjustTitleText(name, Adapt.screenW() - 47 - 73),
                    overflow: TextOverflow.visible,
                    softWrap: false,
                    textAlign: TextAlign.justify,
                    style: TextStyle(fontSize: 18, color: Colors.white),
                  ),
                ),
              ),
            ],
          ),
        ),
        Container(
          margin: const EdgeInsets.only(top: 0, left: 0, right: 0, bottom: 0),
//          color: Colors.transparent,
          child: PureTextWidget(
            text: desc ?? "",
            width: Adapt.screenW() - 15 - 72,
            style: TextStyle(
              color: Colors.white,
              fontSize: 16,
              letterSpacing: 0,
              wordSpacing: 0,
            ),
            maxLines: 5,
          ),
//          child: Text(
//            desc ?? "",
//            maxLines: 5,
//            overflow: TextOverflow.ellipsis,
//            style: TextStyle(
//              fontSize: 16,
////              fontWeight: FontWeight.w400,
//              color: Colors.white,
//            ),
//          ),
        ),
      ],
    );
  }

//底部评论框
  Widget normalCommentButtonStack(String commentStr) => Stack(
        overflow: Overflow.clip,
        children: [
          Container(
            height: Adapt.onepx(),
            margin: EdgeInsets.only(top: 0),
            decoration: BoxDecoration(
                color: Colors.white,
                border: Border.all(
                    color: Colors.transparent, width: Adapt.onepx())),
          ),
          Container(
            height: 48,
            margin: const EdgeInsets.only(left: 0, right: 0, top: 1),
            color: Colors.transparent,
            child: Row(
              crossAxisAlignment: CrossAxisAlignment.center,
              children: [
                Container(
                  width: 16,
                  height: 19,
                  margin: const EdgeInsets.fromLTRB(0.5, 8, 5, 10),
                  child: Image.asset(
                    'assets/images/video_write_comment_icon.png',
                    fit: BoxFit.cover,
                  ),
                ),
                Expanded(
                  child: Container(
                    height: 48,
                    color: Colors.transparent,
                    margin: const EdgeInsets.only(
                        top: 0, left: 0, right: 0, bottom: 0),
                    alignment: Alignment.centerLeft,
                    child: Text(
                      commentStr != null ? commentStr : '写评论...',
                      textAlign: TextAlign.left,
                      maxLines: 1,
                      overflow: TextOverflow.ellipsis,
                      style: TextStyle(
                        fontSize: 16,
                        color: Colors.white,
                      ),
                    ),
                  ),
                ),
              ],
            ),
          ),
        ],
      );

//仅查看详情的广告视频的底部视图
  Widget viedoAdSingleDetailWiget(BuildContext context, String url, String name,
      String desc, String detailUrl) {
    bool isUrlValid = UITools.checkUrlStringIsValid(url);
    return Container(
//      height: 215,
      color: Colors.transparent,
      child: Column(
        mainAxisAlignment: MainAxisAlignment.spaceAround,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          GestureDetector(
            child: Container(
//            color: Colors.red,
              alignment: Alignment.bottomLeft,
              margin: EdgeInsets.only(left: 15, right: 15, bottom: 15),
              child: Flex(
                direction: Axis.horizontal,
                children: <Widget>[
                  //用户头像
                  Container(
                    width: isUrlValid ? 25 : 0,
                    height: 25,
//                  color: Colors.amberAccent,
                    margin: EdgeInsets.only(left: 0),
                    child: url != null
                        ? Container(
                            margin: EdgeInsets.all(1),
                            decoration: BoxDecoration(
                              color: Colors.white,
                              shape: BoxShape.rectangle,
                              borderRadius: BorderRadius.circular(11.5),
                              image: DecorationImage(
                                fit: BoxFit.cover,
                                image: new FadeInImage.assetNetwork(
                                  placeholder:
                                      ('assets/images/video_user_default_icon.png'),
                                  image: url,
                                ).image,
                              ),
                            ),
                          )
                        : Container(),
                    decoration: BoxDecoration(
                      color: url != null ? Colors.white : Colors.transparent,
                      shape: BoxShape.rectangle,
                      borderRadius: BorderRadius.circular(12.5),
                    ),
                  ),
                  Expanded(
                    child: Container(
                      padding:
                          EdgeInsets.fromLTRB((isUrlValid ? 7 : 0), 0, 0, 0),
                      height: 25,
                      child: Text(
                        //用户昵称
                        getAdjustTitleText(name, Adapt.screenW() - 47 - 71),
                        overflow: TextOverflow.ellipsis,
                        style: TextStyle(fontSize: 18, color: Colors.white),
                      ),
                    ),
                  ),
                ],
              ),
            ),
            onTap: () {
              ILog.lg.i("点击来源和头像");
              clickVideoAdDetailButtonWidget(context, AdClickAreaType.from);
            },
          ),
          GestureDetector(
            child: Container(
                //广告视频描述文本
                width: Adapt.screenW() - 20,
//                    height: 120,
//              color: Colors.transparent,
                margin: const EdgeInsets.only(
                    top: 0, left: 15, right: 15, bottom: 10),
                child: TextAdWidget(
                  text: desc ?? "",
                  width: Adapt.screenW() - 30,
                  style: TextStyle(
                    color: Colors.white,
                    fontSize: 15,
                    letterSpacing: 0.5,
                    wordSpacing: 1,
                  ),
                  maxLines: 5,
                )),
            onTap: () {
              ILog.lg.i("点击标题");
              clickVideoAdDetailButtonWidget(context, AdClickAreaType.title);
            },
          ),
          Container(
            //查看详情按钮
//              width: 100,
            height: 34,
            color: Colors.transparent,
            alignment: Alignment.center,
            margin: EdgeInsets.only(
                left: 15,
                right: 15,
                bottom: UITools.getAdBottomMargin(context),
                top: 0),
            padding: EdgeInsets.all(0),
            child: new Padding(
              padding: new EdgeInsets.fromLTRB(0.0, 0.0, 0.0, 0.0),
              child: new Row(
                children: <Widget>[
                  new Expanded(
                    child: new FlatButton(
                      onPressed: () {
                        print("查看详情");
                        clickVideoAdDetailButtonWidget(
                            context, AdClickAreaType.detail);
                      },
                      //通过控制 Text 的边距来控制控件的高度
                      textColor: Colors.white,
                      child: new Padding(
                        padding: new EdgeInsets.fromLTRB(0.0, 0.0, 0.0, 0.0),
                        child: new Text(
                          "查看详情",
                          style: TextStyle(fontSize: 15, letterSpacing: 1),
                        ),
                      ),
                      color: Color(0xFFFF7237),
                      //Colors.deepOrange,
                      shape: RoundedRectangleBorder(
                        borderRadius: BorderRadius.all(Radius.circular(2)),
                      ),
                    ),
                  ),
                ],
              ),
            ),
          ),
        ],
      ),
    );
  }

  //查看详情和下载APP的广告视频的底部视图
  Widget viedoAdDetailAndDownloadWiget(BuildContext context, String url,
      String name, String desc, String detailUrl, String appUrl) {
    bool isUrlValid = UITools.checkUrlStringIsValid(url);
    return Container(
//      height: 215,
      color: Colors.transparent,
      child: Column(
        mainAxisAlignment: MainAxisAlignment.spaceAround,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          GestureDetector(
            child: Container(
              //用户信息区
//            color: Colors.transparent,
              alignment: Alignment.bottomLeft,
              margin: EdgeInsets.only(left: 15, right: 15, bottom: 15),
              child: Flex(
                direction: Axis.horizontal,
                children: <Widget>[
                  //用户头像
                  Container(
                    width: isUrlValid ? 25 : 0,
                    height: 25,
//                  color: Colors.amberAccent,
                    margin: EdgeInsets.only(left: 0),
                    child: url != null
                        ? Container(
                            margin: EdgeInsets.all(1),
                            decoration: BoxDecoration(
                              color: Colors.white,
                              shape: BoxShape.rectangle,
                              borderRadius: BorderRadius.circular(11.5),
                              image: DecorationImage(
                                fit: BoxFit.cover,
                                image: new FadeInImage.assetNetwork(
                                  placeholder:
                                      ('assets/images/video_user_default_icon.png'),
                                  image: url,
                                ).image,
                              ),
                            ),
                          )
                        : Container(),
                    decoration: BoxDecoration(
                      color: url != null ? Colors.white : Colors.transparent,
                      shape: BoxShape.rectangle,
                      borderRadius: BorderRadius.circular(12.5),
                    ),
                  ),
                  Expanded(
                      child: Container(
                    padding: EdgeInsets.fromLTRB((isUrlValid ? 7 : 0), 0, 0, 0),
                    height: 25,
                    child: Text(
                      //用户昵称
                      getAdjustTitleText(name, Adapt.screenW() - 47 - 71),
                      overflow: TextOverflow.ellipsis,
                      style: TextStyle(fontSize: 18, color: Colors.white),
                    ),
                  )),
                ],
              ),
            ),
            onTap: () {
              ILog.lg.i("点击来源");
              clickVideoAdDetailButtonWidget(context, AdClickAreaType.from);
            },
          ),
          GestureDetector(
            child: Container(
                //广告视频描述文本
                width: Adapt.screenW() - 20,
//              color: Colors.transparent,
                margin: const EdgeInsets.only(
                    top: 0, left: 15, right: 15, bottom: 10),
                child: TextAdWidget(
                  text: desc ?? "",
                  width: Adapt.screenW() - 30,
                  style: TextStyle(
                    color: Colors.white,
                    fontSize: 15,
                    letterSpacing: 0.5,
                    wordSpacing: 1,
                  ),
                  maxLines: 5,
                )),
            onTap: () {
              ILog.lg.i("点击标题");
              clickVideoAdDetailButtonWidget(context, AdClickAreaType.title);
            },
          ),
          Container(
              //底部按钮区域
              color: Colors.transparent,
              height: 34,
              margin: EdgeInsets.only(
                  left: 15,
                  right: 15,
                  bottom: UITools.getAdBottomMargin(context),
                  top: 0),
              child: Flex(
                direction: Axis.horizontal,
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                crossAxisAlignment: CrossAxisAlignment.center,
                children: <Widget>[
                  new Flexible(
                    flex: 1,
                    fit: FlexFit.loose,
                    child: new Padding(
                      padding: new EdgeInsets.fromLTRB(0.0, 0.0, 0.0, 0.0),
                      child: new Row(
                        children: <Widget>[
                          new Expanded(
                            child: new FlatButton(
                              onPressed: () {
                                print("立即下载");
                                clickVideoAdDownloadRightNowButtonWidget(
                                    context, appUrl);
                              },
                              //通过控制 Text 的边距来控制控件的高度
                              textColor: Colors.white,
                              child: new Padding(
                                padding:
                                    new EdgeInsets.fromLTRB(0.0, 0.0, 0.0, 0.0),
                                child: new Text(
                                  "立即下载",
                                  style:
                                      TextStyle(fontSize: 15, letterSpacing: 1),
                                ),
                              ),
                              color: Color(0xFFFF7237),
                              //Colors.deepOrange,
                              shape: RoundedRectangleBorder(
                                borderRadius:
                                    BorderRadius.all(Radius.circular(2)),
                              ),
                            ),
                          ),
                        ],
                      ),
                    ),
                  ),
                ],
              )),
        ],
      ),
    );
  }

//查看详情
  void clickVideoAdDetailButtonWidget(
      BuildContext context, AdClickAreaType clickType) {
    ILog.lg.i("广告详情URl");

    VideoItemModel.of(context).applyPause();

    Map param = new Map();
    param["url"] = _videoItem.videoItem.url ?? "";
//    ILog.lg.i("广告URL="+_videoItem.videoItem.url.toString()+"clickUrl:"+_videoItem.videoItem.clickUrls.toString());
    List<dynamic> clickUrls =
        _videoItem.videoItem.clickUrls ?? List<dynamic>(); //广告点击回传
    param["clickUrls"] = clickUrls;
    SgOperation.openAdDetail(param);

    //pingback
    String pingbackKey = "ShortVideoAdClick";
    Map pingbackParam = new Map();
    pingbackParam["name"] = Constant.CHANNEL ?? "";
    pingbackParam["adid"] = _videoItem.videoItem.newsid ?? ""; //广告id需要根据数据来定
    pingbackParam["title"] = _videoItem.videoItem.title ?? "";
    pingbackParam["url"] = _videoItem.videoItem.url ?? "";
    pingbackParam["type"] = "h5"; //需要区分广告类型 //download带下载功能的广告
//    if(clickType == AdClickAreaType.title || clickType == AdClickAreaType.from || clickType == AdClickAreaType.detail){
//      pingbackParam["clickarea"] = "detail";
//    }
    pingbackParam["clickarea"] = "detail"; //download 需要更加数据来判断
    pingbackParam["h5url"] = _videoItem.videoItem.url ?? "";
    pingbackParam["downloadurl"] = ""; //待修改
    Map dataMap = new Map();
    dataMap["key"] = pingbackKey;
    dataMap["param"] = pingbackParam;
    SgOperation.statistics(SgOperation.STATISTICS_IMMEDIATELY_TYPE, dataMap);
  }

//立即下载
  void clickVideoAdDownloadRightNowButtonWidget(
      BuildContext context, String adUrl) {
    ILog.lg.i("广告立即下载URl:${adUrl}");
    ILog.lg.invoke();

    if (Platform.isIOS) {
      VideoItemModel.of(context).applyPause();
    }

    Map param = new Map();
    param["url"] = _videoItem.videoItem.url ?? "";
    List<dynamic> clickUrls =
        _videoItem.videoItem.clickUrls ?? List<dynamic>(); //广告点击回传
    param["clickUrls"] = clickUrls;
    SgOperation.downloadAdApp(param);

    //pingback
    String pingbackKey = "ShortVideoAdClick";
    Map pingbackParam = new Map();
    pingbackParam["name"] = Constant.CHANNEL ?? "";
    pingbackParam["adid"] = _videoItem.videoItem.newsid ?? ""; //广告id需要根据数据来定
    pingbackParam["title"] = _videoItem.videoItem.title ?? "";
    pingbackParam["url"] = _videoItem.videoItem.url ?? "";
    pingbackParam["type"] = "download";
    pingbackParam["clickarea"] = "download"; //download 需要更加数据来判断
    pingbackParam["h5url"] = "";
    pingbackParam["downloadurl"] = _videoItem.videoItem.url ?? "";
    Map dataMap = new Map();
    dataMap["key"] = pingbackKey;
    dataMap["param"] = pingbackParam;
    SgOperation.statistics(SgOperation.STATISTICS_IMMEDIATELY_TYPE, dataMap);
  }
}
