import 'package:flutter/material.dart';
import 'package:scoped_model/scoped_model.dart';
import 'package:short_video/base/i_log.dart';
import 'package:short_video/model_item.dart';
import 'package:short_video/platform/sg_player.dart';
import 'package:short_video/widget/error_widget.dart';
import 'package:short_video/widget/givelike_anim_screen.dart';
import 'package:short_video/widget/page_control_widget.dart';
import 'package:short_video/widget/touch_event_screen.dart';

/// 单条视频展示页面
class VideoPageItem extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    print("TIME- VideoPageItem::build");
    return ScopedModelDescendant<VideoItemModel>(
        builder: (context, child, model) {
      List<Widget> _contentList = [];

      if (model.showPlayer) {
        VideoPlayerController vpc = model.playerController;
        // 当前播放页
        final Size size = vpc.value.size;
        _contentList.add(Center(
          child: AspectRatio(
            aspectRatio: size.width / size.height,
            child: VideoPlayer(vpc),
          ),
        ));

        // 手势处理
        _contentList.add(TouchEventScreen());

        // 暂停键
        if (model.pauseVisible) {
          var pauseBt = Center(
            child: new FlatButton(
              highlightColor: Colors.transparent,
              splashColor: Colors.transparent,
              child: Image(
                width: 80,
                image: Image.asset("assets/images/video_play_btn_normal.png")
                    .image,
              ),
              onPressed: () {
                model.setPauseOrResume();
              },
            ),
          );
          _contentList.add(pauseBt);
//          _contentList.add(FadeInWidgetSwitcher(child: pauseBt));
        }
      }

      // 播放控制控件等
      _contentList.add(VideoItemMaskWidget(model));

      // 出错
      if (model.isNoNet) {
        _contentList.add(
          VideoPageError(VideoPageError.STATE_NONET, model.onClickErrorRetry),
        );
      } else if (model.isError) {
        _contentList.add(
          VideoPageError(VideoPageError.STATE_ERROR, model.onClickErrorRetry),
        );
      } else if (model.isLoading) {
        // 加载中
        _contentList.add(
          VideoPageError(VideoPageError.STATE_LOADING, model.onClickErrorRetry),
        );
      } else {
        // 添加动画图层
        if(model.heartAnimNotifier != null){
          _contentList.add(IgnorePointer(
            ignoring: true,
            child: GiveLikeAnimationScreen(model.heartAnimNotifier),
          ));
        }
      }
      return Stack(
        alignment: AlignmentDirectional.topStart,
        children: _contentList,
      );
    });
  }
}
