import 'package:flutter/material.dart';
import 'dart:ui';
import 'dart:math';

import 'package:short_video/base/i_log.dart';

//该类是用来展示更多菜单的route
class VideoPopRoute extends PopupRoute{
  final Duration _duration = Duration(milliseconds: 200);
  Widget child;

  VideoPopRoute({@required this.child});

  @override
  Color get barrierColor => null;

  @override
  bool get barrierDismissible => true;

  @override
  String get barrierLabel => null;

  @override
  Widget buildPage(BuildContext context, Animation<double> animation, Animation<double> secondaryAnimation) {
    return child;
  }
  @override
  Duration get transitionDuration => _duration;

}



class VideoPopup extends StatelessWidget{
  final Widget child;
  final Function onClick; //点击child事件
  final Function backgroundClick; //点击child事件
  final double left; //距离左边位置
  final double top; //距离上面位置

  VideoPopup({
    @required this.child,
    this.onClick,
    this.backgroundClick,
    this.left,
    this.top,
  });

  @override
  Widget build(BuildContext context) {
    return Material(color: Colors.transparent, child: GestureDetector(child: Stack(
      children: <Widget>[
        Container(width: MediaQuery.of(context).size.width, height: MediaQuery.of(context).size.height, color: Colors.transparent,),
        Positioned(child: GestureDetector(child: child, onTap: (){ //点击子child
          ILog.lg.i("点击事件：onClick="+onClick.toString()+"context->"+context.toString());
          if(onClick != null){
            Navigator.of(context).pop();
            onClick();
          }
        }),
          left: left,
          top: top,),
      ],
    ),
      onTap: (){ //点击空白处
        Navigator.of(context).pop();
        print("点击VideoPopup背景：context->"+context.toString());
        if(backgroundClick != null) {
          backgroundClick();
        }
      },
    ),);
  }
}

//新建类继承于CustomPainter并且实现CustomPainter里面的paint（）和shouldRepaint方法
class PopupMenuBorderPainter extends CustomPainter {
  /*Paint _paint = Paint()
    ..color = Colors.blueAccent //画笔颜色
    ..strokeCap = StrokeCap.round //画笔笔触类型
    ..isAntiAlias = true //是否启动抗锯齿
    ..blendMode = BlendMode.exclusion //颜色混合模式
    ..style = PaintingStyle.fill //绘画风格，默认为填充
    ..colorFilter = ColorFilter.mode(Colors.blueAccent,
        BlendMode.exclusion) //颜色渲染模式，一般是矩阵效果来改变的,但是flutter中只能使用颜色混合模式
    ..maskFilter =
    MaskFilter.blur(BlurStyle.inner, 3.0) //模糊遮罩效果，flutter中只有这个
    ..filterQuality = FilterQuality.high //颜色渲染模式的质量
    ..strokeWidth = 15.0 ;//画笔的宽度
      */

  Paint _paint = new Paint()
    ..color = Color(0xFFF8F8F8)
    ..strokeJoin = StrokeJoin.bevel
    ..strokeCap = StrokeCap.square
    ..isAntiAlias = true
    ..strokeWidth = 2.0
    ..style = PaintingStyle.fill;
  Paint leftPaint = new Paint()
    ..color = Colors.white
    ..strokeJoin = StrokeJoin.bevel
    ..strokeCap = StrokeCap.square
    ..isAntiAlias = true
    ..strokeWidth = 2.0
    ..style = PaintingStyle.fill;

  ///Flutter中负责View绘制的地方，使用传递来的canvas和size即可完成对目标View的绘制
  @override
  void paint(Canvas canvas, Size size) {
    double widthTemp = size.width;
    double heighTemp = size.height;
    double arrowHeight = 8;
    double arrowWidth = 16;
    double arrowPaddingRightX = 20;
    double paddingx = 0;
    double joinRadius = 4.0;
    double leftSectionWidth = 40;
    //绘制底部带边框的背景视图
    Path path = new Path();
    path.moveTo(paddingx, joinRadius);
    Rect rect = Rect.fromCircle(center: Offset(paddingx+joinRadius, arrowHeight+joinRadius), radius: joinRadius);
    path.arcTo(rect, 1*pi, pi/2,false);
    path.lineTo(widthTemp-arrowPaddingRightX-arrowWidth,arrowHeight);
    path.lineTo(widthTemp-arrowPaddingRightX-(arrowWidth/2.0), 0.0);
    path.lineTo(widthTemp-arrowPaddingRightX, arrowHeight);
    path.lineTo(widthTemp-joinRadius, arrowHeight);
    rect = Rect.fromCircle(center: Offset(widthTemp-joinRadius, arrowHeight+joinRadius), radius: joinRadius);
    path.arcTo(rect, 3*pi/2, pi/2,false);
    path.lineTo(widthTemp, arrowHeight+joinRadius);
    path.lineTo(widthTemp, heighTemp-joinRadius);
    rect = Rect.fromCircle(center: Offset(widthTemp-joinRadius, heighTemp-joinRadius), radius: joinRadius);
    path.arcTo(rect, 0, pi/2,false);
    path.lineTo(widthTemp-joinRadius, heighTemp);
    path.lineTo(paddingx+joinRadius, heighTemp);
    rect = Rect.fromCircle(center: Offset(paddingx+joinRadius, heighTemp-joinRadius), radius: joinRadius);
    path.arcTo(rect, pi/2, pi/2,false);
    path.lineTo(paddingx, joinRadius+arrowHeight);
    path.close();

    canvas.drawPath(
        path,
        _paint
          ..style = PaintingStyle.fill
    );
    canvas.save();
    //绘制左侧白色的部分
    Path pathLeft = new Path();
    pathLeft.moveTo(paddingx, joinRadius);
    rect = Rect.fromCircle(center: Offset(paddingx+joinRadius, arrowHeight+joinRadius), radius: joinRadius);
    pathLeft.arcTo(rect, 1*pi, pi/2,false);
    pathLeft.lineTo(paddingx+leftSectionWidth,arrowHeight);
    pathLeft.lineTo(paddingx+leftSectionWidth, heighTemp);
    pathLeft.lineTo(paddingx+joinRadius, heighTemp);
    rect = Rect.fromCircle(center: Offset(paddingx+joinRadius, heighTemp-joinRadius), radius: joinRadius);
    pathLeft.arcTo(rect, pi/2, pi/2,false);
    pathLeft.lineTo(paddingx, joinRadius+arrowHeight);
    pathLeft.close();

    canvas.drawPath(
        pathLeft,
        leftPaint
          ..color = Colors.white
          ..style = PaintingStyle.fill
    );
    canvas.restore();

  }

  ///控制自定义View是否需要重绘的，返回false代表这个View在构建完成后不需要重绘。
  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return false;
  }
}