import 'package:flutter/material.dart';
import 'dart:ui';

//纯文本的文本widget
class PureTextWidget extends StatelessWidget {
  String text;
  TextStyle style;
  double width;
  int maxLines;
  PureTextWidget(
      {
        @required this.text,
        @required this.style,
        @required this.width,
        @required this.maxLines,

      });
  double height = 0;
  @override
  Widget build(BuildContext context) {
    double height = MultiRowPureTextPainter.findTextLabelHeight(this.text, this.style, this.width,this.maxLines);
//    print('文本框高度height='+height.toString());
    return CustomPaint(
      size: Size(width, height),
      painter: MultiRowPureTextPainter(
          textStyle: this.style,
          text: this.text,
          maxLines: this.maxLines,
          width: width
      ),
    );
  }
}


/// 该类是用来绘制描述文本
class MultiRowPureTextPainter extends CustomPainter {
  String text;//文本
  double width;//宽度
  int maxLines;//最大行数限制
  TextStyle textStyle;//样式
  double height;
  MultiRowPureTextPainter(
      {
        @required this.text,
        @required this.textStyle,
        @required this.width,
        @required this.maxLines,
        this.height
      });
  @override
  void paint(Canvas canvas, Size size) {
    var paint = new Paint();
    paint.color = textStyle.color;

    if(this.text == null || this.text.length < 1){
      TextSpan adSpan = new TextSpan(
        style: textStyle,
        text: '',
      );
      TextPainter adTp = new TextPainter(
        text: adSpan,
        textAlign: TextAlign.center,
        textDirection: TextDirection.ltr,
      );
      adTp.layout();
      Offset offsetTemp= Offset(0, 3);
      adTp.paint(canvas, offsetTemp);
      return;
    }

    double offsetX = 0;
    double offsetY = 0;
    bool newLine = false;
    bool finish = false;
    double ellipsisWidth = findeTextLabelWidth("...", this.textStyle);
    int lineNumber = 1;
    int textLineNumber = findTextLabelLineNumber(this.text, this.textStyle,this.width);
    int totalWords = 0;//总共字符数目
    text.runes.forEach((str) {
      totalWords = totalWords + 1;
    });
    int curWords = 0;//当前已绘字符数

    TextSpan zhongSpan = new TextSpan(style: this.textStyle, text: "中");
    TextPainter zhongTp = new TextPainter(
        text: zhongSpan,
        textAlign: TextAlign.center,
        textDirection: TextDirection.ltr
    );
    zhongTp.layout();
    double zhongHeight = zhongTp.height;//中文高度

    text.runes.forEach((rune){
      bool stop = false;
      int cur = text.length;
      String str = new String.fromCharCode(rune);
      TextSpan span = new TextSpan(style: textStyle, text: str);
      TextPainter tp = new TextPainter(
          text: span,
          textAlign: TextAlign.center,
          textDirection: TextDirection.ltr
      );
      tp.layout();
      Offset offsetTemp;
      if(textLineNumber <= this.maxLines) {//行数可以完全展示
        curWords = curWords + 1;
        if(offsetX + tp.width > this.width) {
          newLine = true;
        }else {
          newLine = false;
        }
//        print("当前字符"+str+"中文高度"+zhongHeight.toString()+"字符高度="+tp.height.toString());
        if(newLine) {
          offsetX = 0;
          lineNumber = lineNumber + 1;
          offsetY += zhongHeight;// tp.height;
          newLine = false;
          offsetTemp = new Offset(offsetX, offsetY+(zhongHeight-tp.height)/2.0);
          offsetX = tp.width;
        }else {
          offsetTemp = new Offset(offsetX, offsetY+(zhongHeight-tp.height)/2.0);
          offsetX = offsetX + tp.width;
        }
        if(lineNumber > this.maxLines || curWords > totalWords) {
          stop = true;
        }
        if(!stop){//未结束继续绘制
          tp.paint(canvas, offsetTemp);
        }
      }else {//文字数目超过行数的显示
//        print("当前字符"+str+"中文高度"+zhongHeight.toString()+"字符高度="+tp.height.toString());
        if(offsetX + tp.width > this.width) {
          newLine = true;
        }else {
          newLine = false;
        }
        if(newLine) {
          offsetX = 0;
          lineNumber = lineNumber + 1;
          offsetY += zhongHeight; //tp.height;
          newLine = false;
          offsetTemp = new Offset(offsetX, offsetY+(zhongHeight-tp.height)/2.0);
          offsetX = tp.width;
        }else {
          offsetTemp = new Offset(offsetX, offsetY+(zhongHeight-tp.height)/2.0);
          offsetX = offsetX + tp.width;
        }
        if(lineNumber >= this.maxLines) {
          if(offsetX + ellipsisWidth > this.width  && !finish) {
            stop = true;
            //当前字符不能继续绘制,需要绘制'...'
            TextSpan ellipsisSpan = new TextSpan(
              style:this.textStyle,
              text:'...',
            );
            TextPainter ellipsisTp = new TextPainter(
              text: ellipsisSpan,
              textAlign: TextAlign.center,
              textDirection: TextDirection.ltr,
            );
            ellipsisTp.layout();
            ellipsisTp.paint(canvas, offsetTemp);
            offsetX = offsetTemp.dx + ellipsisTp.width;
            finish = true;//结束
            return;
          }else {
            if(finish) {
              return;
            }
            tp.paint(canvas, offsetTemp);
          }
        }else {//未达到最大行数继续绘制
          tp.paint(canvas, offsetTemp);
        }
      }
    });
  }
  double findMaxWidth(String text, TextStyle style) {
    double maxWidth = 0;
    text.runes.forEach((rune) {
      String str = new String.fromCharCode(rune);
      TextSpan span = new TextSpan(style: style, text: str);
      TextPainter tp = new TextPainter(
          text: span,
          textAlign: TextAlign.center,
          textDirection: TextDirection.ltr);
      tp.layout();
      maxWidth = max(maxWidth, tp.width);
    });

    return maxWidth;
  }
//查询文本宽度
  double findeTextLabelWidth(String text, TextStyle style) {
    double adWidth = 0;
    text.runes.forEach((rune){
      String str = new String.fromCharCode(rune);
      TextSpan span = new TextSpan(style: style, text: str);
      TextPainter tp = new TextPainter(
          text: span,
          textAlign: TextAlign.center,
          textDirection: TextDirection.ltr
      );
      tp.layout();
      adWidth = adWidth + tp.width;
    });
    return adWidth;
  }

  //查询文本的行数
  static int findTextLabelLineNumber(String text, TextStyle style ,double width) {
    double offsetX = 0;//坐标X
    double offsetY = 0;//坐标Y
    bool newLine = false;
    int lineNumber = 1;//行数统计
    text.runes.forEach((rune){
      String str = new String.fromCharCode(rune);
      TextSpan span = new TextSpan(style: style, text: str);
      TextPainter tp = new TextPainter(
          text: span,
          textAlign: TextAlign.center,
          textDirection: TextDirection.ltr
      );
      tp.layout();
      if(offsetX + tp.width > width) {
        newLine = true;
      }else {
        newLine = false;
      }
      if(newLine) {
        offsetX = 0;
        lineNumber = lineNumber + 1;
        offsetY += tp.height;
        newLine = false;
        offsetX = tp.width;
      }else {
        offsetX = offsetX + tp.width;
      }
    });
//    print('最终行数='+lineNumber.toString());
    return lineNumber;
  }
  //查询文本的高度
  static double findTextLabelHeight(String text, TextStyle style ,double width,int maxLines) {
    double offsetX = 0;//坐标X
    double offsetY = 0;//坐标Y
    bool newLine = false;
    int lineNumber = 1;//行数统计
    double textHeight = 0;

    text.runes.forEach((rune){
      String str = new String.fromCharCode(rune);
      TextSpan span = new TextSpan(style: style, text: str);
      TextPainter tp = new TextPainter(
          text: span,
          textAlign: TextAlign.center,
          textDirection: TextDirection.ltr
      );
      tp.layout();
//      print("字符="+str);
      if(offsetX + tp.width > width) {
        newLine = true;
      }else {
        newLine = false;
      }
      if(newLine) {
        offsetX = 0;
        lineNumber = lineNumber + 1;
        offsetY += tp.height;
        newLine = false;
        offsetX = tp.width;
      }else {
        offsetX = offsetX + tp.width;
      }
    });
    TextSpan span = new TextSpan(style: style, text: "中");
    TextPainter tp = new TextPainter(
        text: span,
        textAlign: TextAlign.center,
        textDirection: TextDirection.ltr
    );
    tp.layout();
    maxLines = maxLines <= lineNumber? maxLines:lineNumber;
    textHeight = maxLines.toDouble() * tp.height;
//    print('最终行数='+maxLines.toString()+"；文本行数"+lineNumber.toString());
    return textHeight;
  }

  @override
  bool shouldRepaint(MultiRowPureTextPainter oldDelegate) {//是否重绘
    return oldDelegate.text != text ||
        oldDelegate.textStyle != textStyle ||
        oldDelegate.width != width;
  }
  double max(double a, double b) {
    if (a > b) {
      return a;
    } else {
      return b;
    }
  }
}