import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:short_video/base/i_log.dart';
import 'package:short_video/bean/heart_info.dart';
import 'dart:math';

import 'package:short_video/model_item.dart';
import 'package:short_video/widget/common/positioned_tap_detector.dart';

/// 处理打击，双击等用户事件；
/// 主要为完成快速点击，生成心形动画
// ignore: must_be_immutable
class TouchEventScreen extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    Widget contentWidget = Container(
      constraints: BoxConstraints.expand(),
      decoration: BoxDecoration(
          gradient: LinearGradient(
        colors: [
          Color.fromRGBO(0, 0, 0, 0.5),
          Color.fromRGBO(0, 0, 0, 0),
          Color.fromRGBO(0, 0, 0, 0),
          Color.fromRGBO(0, 0, 0, 0.5)
        ],
        begin: Alignment.topCenter,
        end: Alignment.bottomCenter,
        tileMode: TileMode.repeated,
      )),
    );

    Widget res = PositionedTapDetector(
      behavior: HitTestBehavior.translucent,
      onTap: (position) {
        ILog.lg.invoke("PositionedTapDetector::onTap:$position");
        _onSingleClick(context);
      },
      onDoubleTap: (position) {
        ILog.lg.invoke("PositionedTapDetector::onDoubleTap:${position.global}");
        _onDoubleClick(context, position.global);
      },
      child: contentWidget,
    );

    return res;
  }

  void _onSingleClick(BuildContext context) {
    VideoItemModel.of(context).setPauseOrResume();
  }

  void _onDoubleClick(BuildContext context, Offset off) {
    Offset pos = (context.findRenderObject() as RenderBox).globalToLocal(off);
    final random = new Random();
    int randomInt = random.nextInt(100);
    double angle;
    if (randomInt <= 33) {
      angle = pi / 9;
    } else if (randomInt > 33 && randomInt <= 66) {
      angle = 0;
    } else {
      angle = -pi / 9;
    }
//    print("随机数="+randomInt.toString()+"角度="+angle.toString());
    HeartInfo hi =
        HeartInfo(DateTime.now().millisecondsSinceEpoch, pos.dx, pos.dy, angle);
    VideoItemModel.of(context).onDoubleClickGiveLike(hi);
  }
}
