import 'package:flutter/material.dart';
import 'package:short_video/model_item.dart';
import 'package:short_video/platform/sg_operation.dart';
import 'package:short_video/model_main.dart';

class WriteCommentWidget extends StatelessWidget {
  int commentStayTime;
  final String channelId;
  DateTime _lastClickTime;

  WriteCommentWidget(this.channelId);

  double getAppBottomMargin(BuildContext context) {
    double bottomPadding = MediaQuery.of(context).padding.bottom;
    if (bottomPadding > 10) {
      return 13+20.0;
    } else {
      return 0;
    }
  }

  @override
  Widget build(BuildContext context) {
    double bottomMargin = getAppBottomMargin(context);
    Widget _contentWidget = Stack(
      overflow: Overflow.clip,
      children: [
        Container(
          height: 0.5,
          margin: EdgeInsets.only(top: 0, left: 15, right: 15),
          decoration: BoxDecoration(
              color: Colors.transparent,
              border: Border.all(color: Color(0x7FFFFFFF), width: 0.5)),
        ),
        Container(
          height: 48,
          margin:
              EdgeInsets.only(left: 0, right: 0, top: 1, bottom: bottomMargin),
          color: Colors.transparent,
          child: Row(
            crossAxisAlignment: CrossAxisAlignment.center,
            children: [
              Container(
                width: 16,
                height: 19,
                margin: const EdgeInsets.fromLTRB(15.5, 8, 15, 10),
                child: Image.asset(
                  'assets/images/video_write_comment_icon.png',
                  fit: BoxFit.cover,
                ),
              ),
              Expanded(
                child: Container(
                  height: 48,
                  color: Colors.transparent,
                  margin: const EdgeInsets.only(
                      top: 0, left: 0, right: 0, bottom: 0),
                  alignment: Alignment.centerLeft,
                  child: Text(
                    '写评论...',
//                    commentStr != null ? commentStr : '写评论...',
                    textAlign: TextAlign.left,
                    maxLines: 1,
                    overflow: TextOverflow.ellipsis,
                    style: TextStyle(
                      fontSize: 16,
                      color: Colors.white,
                    ),
                  ),
                ),
              ),
            ],
          ),
        ),
      ],
    );

    return GestureDetector(
      behavior: HitTestBehavior.translucent,
      onTap: () {
        _onClick(context);
      },
      child: _contentWidget,
    );
  }

  void _onClick(BuildContext context) {
    if (_lastClickTime != null &&
        DateTime.now().difference(_lastClickTime) <= Duration(seconds: 1)) {
      return;
    } else {
      _lastClickTime = DateTime.now();
    }
    int _playOrderRecord = MainModel.of(context).getPlayOrder();
    VideoItemModel itemModel = VideoItemModel.of(context);
    MainModel.of(context).setPlayOrder(MainModel.ORDER_TYPE_LOOP_FOR_COMMENTS);
    Map param = new Map();
    param['newsid'] = itemModel.videoItem.newsid ?? "";
    param['url'] = itemModel.videoItem.url ?? "";
    param['title'] = itemModel.videoItem.title ?? "";
    param['input'] = itemModel.commentText ?? "";
    param['key'] = itemModel.videoItem.key ?? "";
    param['from'] = itemModel.videoItem.from ?? "";
    param['topic'] = itemModel.videoItem.topic ?? "";
    param['commentCount'] = itemModel.currCommentCount ?? 0;
//        param['commentStr'] = commentStr;
    SgOperation.writeComments(param).then((value) {
      //恢复播放模式
      MainModel.of(context).setPlayOrder(_playOrderRecord);
      if (value is Map) {
        String input = value["input"];
        int staytime = value["staytime"];
        int commentCount = value["commentCount"];
        assert(staytime != null);
        itemModel.onNativeCommentResult(input, commentCount);
        commentStayTime = staytime;
        //pingback
        String pingbackKey = "InformStayTime";
        Map pingbackParam = new Map();
        pingbackParam["StayModule"] = "shortvideolist";
        pingbackParam["Channel"] = channelId ?? "小视频";
        pingbackParam["page-type"] = "video";
        String urlTemp = itemModel.videoItem.url ?? "";
        Uri u = Uri.parse(urlTemp);
        String newsType = "other";
        if (u.host != null && u.host.toLowerCase().startsWith("bazinga")) {
          newsType = "bazinga";
        }
        pingbackParam["news-type"] = newsType;
        pingbackParam["newsid"] = itemModel.videoItem.newsid;
        pingbackParam["CommentStayTime"] = commentStayTime;
        Map dataMap = new Map();
        dataMap["key"] = pingbackKey;
        dataMap["param"] = pingbackParam;
        SgOperation.statistics(
            SgOperation.STATISTICS_IMMEDIATELY_TYPE, dataMap);
      }
    });
    //pingback
    String pingbackKey = "ShortVideoCommentBoxClick";
    Map pingbackParam = new Map();
    pingbackParam["name"] = channelId ?? "";
    pingbackParam["url"] = itemModel.videoItem.url ?? "";
    pingbackParam["title"] = itemModel.videoItem.title ?? "";
    pingbackParam["newsid"] = itemModel.videoItem.newsid ?? "";
    Map dataMap = new Map();
    dataMap["key"] = pingbackKey;
    dataMap["param"] = pingbackParam;
    SgOperation.statistics(SgOperation.STATISTICS_IMMEDIATELY_TYPE, dataMap);
  }
}
